"""
* genfire.gui.VolumeSlicer *

The volume slicer module


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""

from genfire.gui import VolumeSlicer_MainWindow
from PyQt5 import QtCore, QtGui, QtWidgets
import matplotlib
matplotlib.use("Qt5Agg")
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import numpy as np
from functools import partial
from genfire.gui.utility import toString, toQString, toInt, toFloat


class VolumeSlicer(QtWidgets.QMainWindow):
    def __init__(self, volume):
        super(VolumeSlicer, self).__init__()
        self.volume = volume
        self.ui = VolumeSlicer_MainWindow.Ui_VolumeSlicer()
        self.ui.setupUi(self) # build interface

        self.ui.checkBox_lockcmap.toggled.connect(self.toggleLockCmap)
        self.lockColormap = False
        self.ui.checkBox_lockcmap.setChecked(False)

        # figures for plotting
        self.fig1 = plt.figure()
        self.fig2 = plt.figure()
        self.fig3 = plt.figure()

        # canvases link the Qt widgets with the matplotlib figures
        self.canvas1 = FigureCanvas(self.fig1)
        self.canvas2 = FigureCanvas(self.fig2)
        self.canvas3 = FigureCanvas(self.fig3)

        self.slice1 = self.fig1.add_subplot(111)
        self.slice2 = self.fig2.add_subplot(111)
        self.slice3 = self.fig3.add_subplot(111)

        self.navigationToolbar1 = NavigationToolbar(self.canvas1, self)
        self.navigationToolbar2 = NavigationToolbar(self.canvas2, self)
        self.navigationToolbar3 = NavigationToolbar(self.canvas3, self)

        self.ui.vt_lyt_fig1.addWidget(self.navigationToolbar1)
        self.ui.vt_lyt_fig1.addWidget(self.canvas1)
        self.ui.vt_lyt_fig2.addWidget(self.navigationToolbar2)
        self.ui.vt_lyt_fig2.addWidget(self.canvas2)
        self.ui.vt_lyt_fig3.addWidget(self.navigationToolbar3)
        self.ui.vt_lyt_fig3.addWidget(self.canvas3)

        dimx, dimy, dimz = np.shape(volume)
        ncx,  ncy,  ncz  = dimx//2., dimy//2, dimz//2
        ncx,  ncy,  ncz  = int(ncx), int(ncy), int(ncz)

        # use of partial allows binding functions to arguments to prevent having to avoid extra parameter forwarding
        self.ui.scrlbr_fig1.valueChanged.connect(partial(self.setTextFromSlider,self.ui.lineEdit_scrlbr1))
        self.ui.scrlbr_fig2.valueChanged.connect(partial(self.setTextFromSlider,self.ui.lineEdit_scrlbr2))
        self.ui.scrlbr_fig3.valueChanged.connect(partial(self.setTextFromSlider,self.ui.lineEdit_scrlbr3))

        self.ui.scrlbr_fig1.setValue(ncx)
        self.ui.scrlbr_fig1.setMinimum(0)
        self.ui.scrlbr_fig1.setMaximum(dimx-1)

        self.ui.scrlbr_fig2.setValue(ncy)
        self.ui.scrlbr_fig2.setMinimum(0)
        self.ui.scrlbr_fig2.setMaximum(dimy-1)

        self.ui.scrlbr_fig3.setValue(ncz)
        self.ui.scrlbr_fig3.setMinimum(0)
        self.ui.scrlbr_fig3.setMaximum(dimz-1)

        self.ui.lineEdit_scrlbr1.textChanged.connect(partial(self.setSliderFromText,self.ui.scrlbr_fig1))
        self.ui.lineEdit_scrlbr2.textChanged.connect(partial(self.setSliderFromText,self.ui.scrlbr_fig2))
        self.ui.lineEdit_scrlbr3.textChanged.connect(partial(self.setSliderFromText,self.ui.scrlbr_fig3))

        self.currentSliceX = ncx
        self.currentSliceY = ncy
        self.currentSliceZ = ncz
        self.updateAll()

        self.ui.scrlbr_fig1.valueChanged[int].connect(self.updateSliceX)
        self.ui.scrlbr_fig2.valueChanged[int].connect(self.updateSliceY)
        self.ui.scrlbr_fig3.valueChanged[int].connect(self.updateSliceZ)

        self.clim1 = plt.getp(plt.getp(self.slice1,'images')[0],'clim')
        self.clim2 = plt.getp(plt.getp(self.slice2,'images')[0],'clim')
        self.clim3 = plt.getp(plt.getp(self.slice3,'images')[0],'clim')

        self.lockColormap = True
        self.ui.checkBox_lockcmap.setChecked(True)


    def updateSliceX(self, nx):
        self.slice1.imshow(np.squeeze(self.volume[nx, :, :]))
        if self.lockColormap:
            plt.setp(plt.getp(self.slice1,'images')[0],'clim',self.clim1)
        self.canvas1.draw()

    def updateSliceY(self, ny):
        self.slice2.imshow(np.squeeze(self.volume[:, ny, :]))
        if self.lockColormap:
            plt.setp(plt.getp(self.slice2,'images')[0],'clim',self.clim2)
        self.canvas2.draw()

    def updateSliceZ(self, nz):
        self.slice3.imshow(np.squeeze(self.volume[:, :, nz]))
        if self.lockColormap:
            plt.setp(plt.getp(self.slice3,'images')[0],'clim',self.clim3)
        self.canvas3.draw()

    def updateAll(self):
        self.updateSliceX(self.currentSliceX)
        self.updateSliceY(self.currentSliceY)
        self.updateSliceZ(self.currentSliceZ)

    def toggleLockCmap(self):
        if self.ui.checkBox_lockcmap.isChecked():
            self.clim1 = plt.getp(plt.getp(self.slice1,'images')[0],'clim')
            self.clim2 = plt.getp(plt.getp(self.slice2,'images')[0],'clim')
            self.clim3 = plt.getp(plt.getp(self.slice3,'images')[0],'clim')
            self.lockColormap = True
        else:
            self.lockColormap = False

    def setSliderFromText(self, slider, text):
        try:
            val = toInt(text)
        except:
            val = 1
        slider.setValue(val)

    def setTextFromSlider(self, lineedit, value):
        lineedit.setText(toQString(value))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'calculateprojectionseries_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CalculateProjectionSeries_Dialog(object):
    def setupUi(self, CalculateProjectionSeries_Dialog):
        CalculateProjectionSeries_Dialog.setObjectName("CalculateProjectionSeries_Dialog")
        CalculateProjectionSeries_Dialog.resize(762, 382)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(CalculateProjectionSeries_Dialog.sizePolicy().hasHeightForWidth())
        CalculateProjectionSeries_Dialog.setSizePolicy(sizePolicy)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(CalculateProjectionSeries_Dialog)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.lineEdit_angleFile = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        self.lineEdit_angleFile.setMinimumSize(QtCore.QSize(500, 0))
        self.lineEdit_angleFile.setObjectName("lineEdit_angleFile")
        self.horizontalLayout_2.addWidget(self.lineEdit_angleFile)
        self.btn_selectAngleFile = QtWidgets.QPushButton(CalculateProjectionSeries_Dialog)
        self.btn_selectAngleFile.setObjectName("btn_selectAngleFile")
        self.horizontalLayout_2.addWidget(self.btn_selectAngleFile)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(CalculateProjectionSeries_Dialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_8.addWidget(self.line)
        self.label_9 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_8.addWidget(self.label_9)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_7 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_5.addWidget(self.label_7)
        self.label_2 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_5.addWidget(self.label_2)
        self.label_8 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_5.addWidget(self.label_8)
        self.horizontalLayout_4.addLayout(self.verticalLayout_5)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.lineEdit_phi = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_phi.sizePolicy().hasHeightForWidth())
        self.lineEdit_phi.setSizePolicy(sizePolicy)
        self.lineEdit_phi.setMaxLength(6)
        self.lineEdit_phi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_phi.setObjectName("lineEdit_phi")
        self.verticalLayout_6.addWidget(self.lineEdit_phi)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lineEdit_thetaStart = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_thetaStart.sizePolicy().hasHeightForWidth())
        self.lineEdit_thetaStart.setSizePolicy(sizePolicy)
        self.lineEdit_thetaStart.setMaxLength(6)
        self.lineEdit_thetaStart.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_thetaStart.setObjectName("lineEdit_thetaStart")
        self.verticalLayout.addWidget(self.lineEdit_thetaStart)
        self.verticalLayout_6.addLayout(self.verticalLayout)
        self.lineEdit_psi = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_psi.sizePolicy().hasHeightForWidth())
        self.lineEdit_psi.setSizePolicy(sizePolicy)
        self.lineEdit_psi.setMinimumSize(QtCore.QSize(10, 20))
        self.lineEdit_psi.setBaseSize(QtCore.QSize(2, 0))
        self.lineEdit_psi.setMaxLength(6)
        self.lineEdit_psi.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_psi.setObjectName("lineEdit_psi")
        self.verticalLayout_6.addWidget(self.lineEdit_psi)
        self.horizontalLayout_8.addLayout(self.verticalLayout_6)
        self.label_4 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_8.addWidget(self.label_4)
        self.horizontalLayout.addLayout(self.horizontalLayout_8)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_9.addItem(spacerItem)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.lineEdit_thetaStep = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_thetaStep.sizePolicy().hasHeightForWidth())
        self.lineEdit_thetaStep.setSizePolicy(sizePolicy)
        self.lineEdit_thetaStep.setMaxLength(6)
        self.lineEdit_thetaStep.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_thetaStep.setObjectName("lineEdit_thetaStep")
        self.horizontalLayout_5.addWidget(self.lineEdit_thetaStep)
        self.label_5 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.verticalLayout_9.addLayout(self.horizontalLayout_5)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_9.addItem(spacerItem1)
        self.horizontalLayout.addLayout(self.verticalLayout_9)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_10.addItem(spacerItem2)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.lineEdit_thetaStop = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_thetaStop.sizePolicy().hasHeightForWidth())
        self.lineEdit_thetaStop.setSizePolicy(sizePolicy)
        self.lineEdit_thetaStop.setMaxLength(6)
        self.lineEdit_thetaStop.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit_thetaStop.setObjectName("lineEdit_thetaStop")
        self.horizontalLayout_7.addWidget(self.lineEdit_thetaStop)
        self.label_6 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_7.addWidget(self.label_6)
        self.verticalLayout_10.addLayout(self.horizontalLayout_7)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_10.addItem(spacerItem3)
        self.horizontalLayout.addLayout(self.verticalLayout_10)
        self.horizontalLayout_4.addLayout(self.horizontalLayout)
        self.verticalLayout_8.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.checkBox_saveAngles = QtWidgets.QCheckBox(CalculateProjectionSeries_Dialog)
        self.checkBox_saveAngles.setObjectName("checkBox_saveAngles")
        self.horizontalLayout_11.addWidget(self.checkBox_saveAngles)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem4)
        self.verticalLayout_8.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.horizontalLayout_6.addWidget(self.label)
        self.lineEdit_outputFilename = QtWidgets.QLineEdit(CalculateProjectionSeries_Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_outputFilename.sizePolicy().hasHeightForWidth())
        self.lineEdit_outputFilename.setSizePolicy(sizePolicy)
        self.lineEdit_outputFilename.setObjectName("lineEdit_outputFilename")
        self.horizontalLayout_6.addWidget(self.lineEdit_outputFilename)
        self.verticalLayout_8.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_8.addLayout(self.horizontalLayout_3)
        self.label_10 = QtWidgets.QLabel(CalculateProjectionSeries_Dialog)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_8.addWidget(self.label_10)
        self.line_2 = QtWidgets.QFrame(CalculateProjectionSeries_Dialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_8.addWidget(self.line_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(CalculateProjectionSeries_Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_8.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.verticalLayout_8)
        self.horizontalLayout_9.addLayout(self.verticalLayout_2)

        self.retranslateUi(CalculateProjectionSeries_Dialog)
        self.buttonBox.accepted.connect(CalculateProjectionSeries_Dialog.accept)
        self.buttonBox.rejected.connect(CalculateProjectionSeries_Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(CalculateProjectionSeries_Dialog)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_angleFile, self.btn_selectAngleFile)
        CalculateProjectionSeries_Dialog.setTabOrder(self.btn_selectAngleFile, self.lineEdit_phi)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_phi, self.lineEdit_thetaStart)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_thetaStart, self.lineEdit_psi)
        CalculateProjectionSeries_Dialog.setTabOrder(self.lineEdit_psi, self.lineEdit_outputFilename)

    def retranslateUi(self, CalculateProjectionSeries_Dialog):
        _translate = QtCore.QCoreApplication.translate
        CalculateProjectionSeries_Dialog.setWindowTitle(_translate("CalculateProjectionSeries_Dialog", "Dialog"))
        self.label_3.setText(_translate("CalculateProjectionSeries_Dialog", "Filename Containing Euler Angles"))
        self.btn_selectAngleFile.setText(_translate("CalculateProjectionSeries_Dialog", "Browse"))
        self.label_9.setText(_translate("CalculateProjectionSeries_Dialog", "Generate Euler Angles"))
        self.label_7.setText(_translate("CalculateProjectionSeries_Dialog", "Phi"))
        self.label_2.setText(_translate("CalculateProjectionSeries_Dialog", "Theta (Tilt)"))
        self.label_8.setText(_translate("CalculateProjectionSeries_Dialog", "Psi"))
        self.label_4.setText(_translate("CalculateProjectionSeries_Dialog", "Start"))
        self.label_5.setText(_translate("CalculateProjectionSeries_Dialog", "Step"))
        self.label_6.setText(_translate("CalculateProjectionSeries_Dialog", "Stop"))
        self.checkBox_saveAngles.setText(_translate("CalculateProjectionSeries_Dialog", "Save Angles"))
        self.label.setText(_translate("CalculateProjectionSeries_Dialog", "Projection Filename"))
        self.label_10.setText(_translate("CalculateProjectionSeries_Dialog", "*.mrc, *.npy, or *.mat"))




"""
* genfire.fileio *

The primary file input/output module for GENFIRE.


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""

import numpy as np

def readVolume(filename, order="C"):
    """
    * readVolume *

    Load volume into a numpy array

    :param filename: filename of volume
    :param order: "C" for row-major order (C-style), "F" for column-major (Fortran style)
    :return:

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.

    """
    import os
    base, ext = os.path.splitext(filename)
    if (ext == ".mrc"):
        return readMRC(filename, order=order)
    elif (ext == ".mat"):
        return readMAT_volume(filename)
    elif (ext == ".npy"):
        return np.load(filename)

def readMAT_volume(filename):

    #wrapper around scipy's loadmat

    import numpy as np
    import scipy.io
    data = scipy.io.loadmat(filename)
    var_name = [key for key in data if not key.startswith("__")]
    if len(var_name) > 1:
        raise IOError("Only 1 variable allowed per .MAT file")
    else:
        return np.array(data[var_name[0]])

def readNPY(filename, dtype=float, order="C"):
    import numpy as np
    return np.load(filename)

def readMRC(filename, dtype=float, order="C"):
    """
    * readMRC *

    Read in a volume in .mrc file format. See http://bio3d.colorado.edu/imod/doc/mrc_format.txt

    :param filename: Filename of .mrc
    :return: NumPy array containing the .mrc data

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.

    """
    import numpy as np
    import struct
    headerIntNumber = 56
    sizeof_int = 4
    headerCharNumber = 800
    sizeof_char = 1
    with open(filename,'rb') as fid:
        int_header = struct.unpack('=' + 'i'*headerIntNumber, fid.read(headerIntNumber * sizeof_int))
        char_header = struct.unpack('=' + 'c'*headerCharNumber, fid.read(headerCharNumber * sizeof_char))
        dimx, dimy, dimz, data_flag= int_header[:4]
        if (data_flag == 0):
            datatype='u1'
        elif (data_flag ==1):
            datatype='i1'
        elif (data_flag ==2):
            datatype='f4'
        elif (data_flag ==3):
            datatype='c'
        elif (data_flag ==4):
            datatype='f4'
        elif (data_flag ==6):
            datatype='u2'
        else:
            raise ValueError("No supported datatype found!\n")

        return np.fromfile(file=fid, dtype=datatype,count=dimx*dimy*dimz).reshape((dimx,dimy,dimz),order=order).astype(dtype)

def writeMRC(filename, arr, datatype='f4', order="C"):
    """
    * writeMRC *

    Write a volume to .mrc file format. See http://bio3d.colorado.edu/imod/doc/mrc_format.txt
    This version is bare-bones and doesn't write out the full header -- just the critical bits and the
    volume itself

    :param filename: Filename of .mrc file to write
    :param arr: NumPy volume of data to write
    :param dtype: Type of data to write


    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved
    """
    import numpy as np

    dimx, dimy, dimz = np.shape(arr)

    if datatype != arr.dtype:
        arr = arr.astype(datatype)
    # int_header = np.zeros(56,dtype='int32') #must be 4-byte ints
    int_header1 = np.zeros(10,dtype='int32') #must be 4-byte ints
    float_header1 = np.zeros(6,dtype='float32') #must be 4-byte ints
    int_header2 = np.zeros(3,dtype='int32') #must be 4-byte ints
    float_header2 = np.zeros(3,dtype='float32') #must be 4-byte ints
    int_header3 = np.zeros(34,dtype='int32') #must be 4-byte ints


    if (datatype == 'u1'):
        data_flag = 0
    elif (datatype =='i1'):
        data_flag = 1
    elif (datatype =='f4'):
        data_flag = 2
    elif (datatype =='c'):
        data_flag = 3
    elif (datatype =='f4'):
        data_flag = 4
    elif (datatype =='u2'):
        data_flag = 6
    else:
        raise ValueError("No supported datatype found!\n")
    int_header1[:4] = (dimx,dimy,dimz,data_flag)
    int_header1[7:10] = (dimx,dimy,dimz)
    float_header1[:3] = 1
    int_header2[:3] = (1, 2, 3)
    float_header2[:3] = np.min(arr), np.max(arr), np.mean(arr)
    char_header = str(' '*800)
    with open(filename,'wb') as fid:
        fid.write(int_header1.tobytes())
        fid.write(float_header1.tobytes())
        fid.write(int_header2.tobytes())
        fid.write(float_header2.tobytes())
        fid.write(int_header3.tobytes())
        fid.write(char_header.encode('UTF-8'))
        fid.write(arr.tobytes(order=order))


def loadProjections(filename):
    """
    * loadProjections *

    Wrapper function for loading in projections of arbitrary (supported) extension

    :param filename: Filename of images to load
    :return: NumPy array containing projections


    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.
    """
    import os
    filename, file_extension = os.path.splitext(filename)
    if file_extension == ".mat":
        print ("Reading projections from MATLAB file.\n")
        # return readMAT_projections(filename + file_extension)
        return readMAT_volume(filename + file_extension)
    elif file_extension == ".tif":
        print ("Reading projections from .tif file.\n")
        return readTIFF_projections(filename + file_extension)
    elif file_extension == ".mrc":
        print ("Reading projections from .mrc file.\n")
        return readMRC(filename + file_extension)
    elif file_extension == ".npy":
        print ("Reading projections from .npy file.\n")
        return readNPY(filename + file_extension)
    else:
        raise Exception('File format %s not supported.', file_extension)

# def readMAT_projections(filename):
#     """
#     * readMAT *
#
#     Read projections from a .mat file
#
#     :param filename: MATLAB file (.mat) containing projections
#     :return: NumPy array containing projections
#
#
#     Author: Alan (AJ) Pryor, Jr.
#     Jianwei (John) Miao Coherent Imaging Group
#     University of California, Los Angeles
#     Copyright 2015-2016. All rights reserved.
#     """
#
#     import scipy.io
#     import numpy as np
#     import os
#     try: #try to open the projections as a stack
#         projections = scipy.io.loadmat(filename)
#         key = None
#         for k in projections.keys():
#             if k[0] != "_":
#                 key = k
#                 break
#
#         projections = np.array(projections[key])
#     except: ## -- figure out where error is thrown
#          #check if the projections are in individual files
#         flag = True
#         filename_base, file_extension = os.path.splitext(filename)
#         projectionCount = 1
#         while flag: #first count the number of projections so the array can be initialized
#             projectionCount = projectionCount
#             nextFile = filename_base + str(projectionCount) + file_extension
#             if os.path.isfile(nextFile):
#                 projectionCount += 1
#             else:
#                 flag = False
#
#
#         ## open first projection to get dimensions
#         pj = scipy.io.loadmat(filename_base + str(1) + file_extension)
#         pj = pj[projections.keys()[0]]
#         dims = np.shape(pj)
#         #initialize projection array
#         projections = np.zeros((dims[0], dims[1], projectionCount),dtype=int)
#
#         #now actually load in the tiff images
#         for projNum in range(projectionCount):
#             nextFile = filename_base + str(projNum) + file_extension
#             pj = scipy.io.loadmat(filename_base + str(projNum) + file_extension)
#             pj = pj[pj.keys()[0]]
#             projections[:, :, projNum] = np.array(pj)
#
#     return projections


def readTIFF_projections(filename):
    """
    * readTIFF *

    Read (possibly multiple) TIFF images into a NumPy array

    :param filename: Name of TIFF file or TIFF file basename to read. If the filename is a base then
    #       the images must begin with the string contained in filename followed by consecutive integers with
    #       no zero padding, i.e. foo1.tiff, foo2.tiff,..., foo275.tiff
    :return: NumPy array containing projections

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.
    """
    import functools
    from PIL import Image
    import os
    import numpy as np
    from multiprocessing import Pool
    try:
        projections = np.array(Image.open(filename))
    except:
        flag = True
        filename_base, file_extension = os.path.splitext(filename)
        projectionCount = 1
        while flag: #first count the number of projections so the array can be initialized
            projectionCount = projectionCount
            nextFile = filename_base + str(projectionCount) + file_extension
            if os.path.isfile(nextFile):
                projectionCount += 1
            else:
                flag = False

        ## open first projection to get dimensions
        dims = np.shape(Image.open(filename_base + str(1) + file_extension))

        #initialize projection array
        projections = np.zeros((dims[0], dims[1], projectionCount),dtype=int)

        pool = Pool(4)
        func = functools.partial(readInTiffProjection, filename_base)
        pj = pool.map(func, range(projectionCount))
        for j  in range(projectionCount):
            projections[:, :, j] = pj[j]
    return projections

def readInTiffProjection(filename_base, fileNumber):
    """
    * readInTiffProjection *

    Reads and returns a single TIFF image as a NumPy array

    :param filename_base: Base filename of TIFF
    :param fileNumber: Image number
    :return: Image in a 2D NumPy array

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.
    """
    from PIL import Image
    import numpy as np
    nextFile = filename_base + str(fileNumber) + '.tif'
    return np.array(Image.open(nextFile))

def loadAngles(filename):
    """
    * loadAngles *

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved.

    :param filename:
    :return:
    """
    import os
    base,ext = os.path.splitext(filename)
    ext = ext.lower()
    if ext == ".txt":
        return np.loadtxt(filename,dtype=float)
    elif ext== ".npy":
        return np.load(filename)
    elif ext==".mat":
        from genfire.fileio import readVolume
        return readVolume(filename)
    else:
        raise IOError("Unsupported file extension \"{}\" for Euler angles".format(ext))


def saveResults(reconstruction_outputs, filename):
    """
    * saveResults *

    Helper function to save results of GENFIRE reconstruction

    :param reconstruction_outputs: dictionary containing reconstruction, reciprocal error (errK), and possible R_free
    :param filename: Output filename

    Author: Alan (AJ) Pryor, Jr.
    Jianwei (John) Miao Coherent Imaging Group
    University of California, Los Angeles
    Copyright 2015-2016. All rights reserved
    """
    import os
    fn, ext = os.path.splitext(filename)
    writeVolume(filename, reconstruction_outputs['reconstruction'])
    np.savetxt(fn+'_errK.txt',reconstruction_outputs['errK'])
    if 'R_free_total' in reconstruction_outputs.keys():
        np.savetxt(fn+'_Rfree_total.txt',reconstruction_outputs['R_free_total'])
        np.savetxt(fn+'_Rfree_bybin.txt',reconstruction_outputs['R_free_bybin'])

def writeVolume(filename, data, order="C"):
    """
    * writeVolume *

    Wrapper volume to file

    :param filename: output filename with valid extension
    :param data: numpy volume to write
    :param order: "C" for row-major order (C-style), "F" for column-major (Fortran style)
    """
    import os
    fn, ext = os.path.splitext(filename)
    if ext == ".mrc":
        writeMRC(filename, arr=data, order=order)
    elif ext == ".npy":
        import numpy as np
        np.save(filename,data)
    elif ext == ".mat":
        import scipy.io
        scipy.io.savemat(filename, {"data":data})
    else:
        raise IOError("Unsupported file extension \"{}\" for volume object".format(ext))



"""
* genfire.gui.utility *

The gui utility function module


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""

from PyQt5 import QtCore, QtGui
import sys

# strings are very different objects in Python 2 and 3, so a little modification of these
# helper functions is necessary dependent on the version
if sys.version_info >=(3,0):
    def toString(string):
        return str(string)
    def toQString(string):
        return str(string)
    def toFloat(string):
        return float(string)
    def toInt(value):
        return int(value)
else: #python 2
    import PyQt5.QtCore
    def toString(string):
        if isinstance(string,QtCore.QString):
            string = unicode(string.toUtf8(),encoding='UTF-8')
        return string
    def toQString(string):
        return PyQt5.QtCore.QString(str(string))
    def toFloat(value):
        if isinstance(value,QtCore.QString):
            return value.toFloat()[0]
        else:
            return float(value)
    def toInt(value):
        if isinstance(value,QtCore.QString):
            return value.toInt()[0]
        else:
            return int(value)



"""
* genfire.gui.launch *

This is the script for launching the GENFIRE GUI


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""


from PyQt5 import QtCore, QtGui, QtWidgets
import matplotlib
matplotlib.use("Qt5Agg")
from genfire.gui import ProjectionCalculator, VolumeSlicer, GENFIRE_MainWindow
import os
import sys
from genfire.reconstruct import ReconstructionParameters
if sys.version_info >= (3,0):
	from genfire.gui import GENFIRE_qrc_py3
else:
	from genfire.gui import GENFIRE_qrc
from genfire.utility import *
from genfire.gui.utility import toString, toQString, toInt, toFloat

class GenfireMainWindow(QtWidgets.QMainWindow):
    stop_threads = QtCore.pyqtSignal()
    def closeEvent(self, QCloseEvent):
        self.stop_threads.emit()
        GF_logger.listener_thread.wait()
        GF_error_logger.listener_thread.wait()
        import matplotlib.pyplot as plt; plt.close("all")
        QCloseEvent.accept()
    def __init__(self):

        ## Superclass constructor
        super(GenfireMainWindow,self).__init__()

        ## Initialize UI
        self.ui = GENFIRE_MainWindow.Ui_GENFIRE_MainWindow()
        self.ui.setupUi(self)

        ## Initialize Reconstruction Parameters
        self.GENFIRE_ReconstructionParameters = ReconstructionParameters()

        ## Initialize file paths in text boxes
        self.ui.lineEdit_results.setText(toQString(os.path.join(os.getcwd(),'results.mrc')))

        self.ui.lineEdit_support.setText(toQString(os.getcwd()))

        self.ui.lineEdit_pj.setText(toQString(os.getcwd()))

        self.ui.lineEdit_io.setText(toQString(os.getcwd()))

        self.ui.lineEdit_angle.setText(toQString(os.getcwd()))


        ## Push Buttons -- connect each to their main function and check if the reconstruction parameters are good each
        ## time a parameter is changed
        self.ui.btn_projections.clicked.connect(self.selectProjectionFile)
        self.ui.btn_projections.clicked.connect(self.checkParameters)

        self.ui.btn_angles.clicked.connect(self.selectAngleFile)
        self.ui.btn_angles.clicked.connect(self.checkParameters)

        self.ui.btn_support.clicked.connect(self.selectSupportFile)
        self.ui.btn_support.clicked.connect(self.checkParameters)

        self.ui.btn_io.clicked.connect(self.selectInitialObjectFile)
        self.ui.btn_io.clicked.connect(self.checkParameters)

        self.ui.btn_reconstruct.setEnabled(False)
        self.ui.btn_reconstruct.clicked.connect(self.startReconstruction)
        self.ui.btn_reconstruct.clicked.connect(self.checkParameters)
        self.ui.btn_reconstruct.setStyleSheet("background-color: rgb(221,0,0)")

        self.ui.btn_displayResults.clicked.connect(self.displayResults)

        ## Line Edits -- connect each to their main function and check if the reconstruction parameters are good each
        ## time a parameter is changed
        self.ui.lineEdit_pj.textChanged.connect(self.GENFIRE_ReconstructionParameters.setProjectionFilename)
        self.ui.lineEdit_pj.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_angle.textChanged.connect(self.GENFIRE_ReconstructionParameters.setAngleFilename)
        self.ui.lineEdit_angle.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_support.textChanged.connect(self.GENFIRE_ReconstructionParameters.setSupportFilename)
        self.ui.lineEdit_support.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_io.textChanged.connect(self.GENFIRE_ReconstructionParameters.setInitialObjectFilename)
        self.ui.lineEdit_io.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_results.textChanged.connect(self.GENFIRE_ReconstructionParameters.setResultsFilename)
        self.ui.lineEdit_results.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_numIterations.setText(toQString("100"))
        self.ui.lineEdit_numIterations.textChanged.connect(self.safeSetNumIterations)
        self.ui.lineEdit_numIterations.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_oversamplingRatio.setText(toQString("3"))
        self.ui.lineEdit_oversamplingRatio.textChanged.connect(self.safeSetOversamplingRatio)
        self.ui.lineEdit_oversamplingRatio.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_interpolationCutoffDistance.setText(toQString("0.7"))
        self.ui.lineEdit_interpolationCutoffDistance.textChanged.connect(self.safeSetInterpDistance)
        self.ui.lineEdit_interpolationCutoffDistance.textChanged.connect(self.checkParameters)

        self.ui.lineEdit_io.setDisabled(True)
        self.ui.lineEdit_io.setStyleSheet("background-color: gray")

        ## Radio Buttons -- default is resolution extension suppression
        self.ui.radioButton_on.setChecked(True)
        self.ui.radioButton_on.toggled.connect(self.selectResolutionExtensionSuppressionState)

        self.ui.radioButton_off.toggled.connect(self.selectResolutionExtensionSuppressionState)

        self.ui.radioButton_extension.toggled.connect(self.selectResolutionExtensionSuppressionState)

        self.ui.radioButton_FFT.setChecked(True)
        self.ui.radioButton_DFT.toggled.connect(self.setGriddingMethod)
        self.ui.radioButton_FFT.toggled.connect(self.setGriddingMethod)

        ## Check Boxes
        self.ui.checkBox_rfree.setChecked(True)
        self.ui.checkBox_rfree.toggled.connect(self.calculateRfree)

        self.ui.checkBox_provide_io.toggled.connect(self.toggleSelectIO)

        self.ui.checkBox_default_support.toggled.connect(self.toggleUseDefaultSupport)
        self.ui.checkBox_default_support.setChecked(True)

        self.ui.checkBox_support_constraint.setChecked(True)
        self.ui.checkBox_positivity_constraint.setChecked(True)
        self.ui.checkBox_support_constraint.toggled.connect(self.toggleSupportConstraint)
        self.ui.checkBox_positivity_constraint.toggled.connect(self.togglePositivityConstraint)
        self.ui.checkBox_multiGridding.toggled.connect(self.toggleMultiGridding)
        self.ui.checkBox_resCircle.toggled.connect(self.toggleResCircle)

        self.ui.action_Create_Support.triggered.connect(self.launchProjectionCalculator)

        self.ui.action_Volume_Slicer.triggered.connect(self.launchVolumeSlicer)

    def safeSetInterpDistance(self):
        try:
            val = toFloat(self.ui.lineEdit_interpolationCutoffDistance.text())
        except:
            val = 1.0
            self.ui.lineEdit_interpolationCutoffDistance.setText('1.0')
        self.GENFIRE_ReconstructionParameters.setInterpolationCutoffDistance(val)

    def safeSetNumIterations(self):
        try:
            val = toInt(self.ui.lineEdit_numIterations.text())
        except:
            val = 1
            self.ui.lineEdit_numIterations.setText('1')
        self.GENFIRE_ReconstructionParameters.setNumberOfIterations(val)

    def safeSetOversamplingRatio(self):
        try:
            val = toInt(self.ui.lineEdit_oversamplingRatio.text())
        except:
            val = 1
            self.ui.lineEdit_oversamplingRatio.setText('1')
        self.GENFIRE_ReconstructionParameters.setOversamplingRatio(val)

    def setNumberOfIterations(self, value):
        number_of_iterations = toInt(value)
        self.GENFIRE_ReconstructionParameters.setNumberOfIterations(number_of_iterations)

    def calculateRfree(self):
        if self.ui.checkBox_rfree.isChecked() == True:
            self.GENFIRE_ReconstructionParameters.calculateRfree = True
        else:
            self.GENFIRE_ReconstructionParameters.calculateRfree = False

    def launchProjectionCalculator(self):
        from functools import partial
        self.GENFIRE_ProjectionCalculator = ProjectionCalculator.ProjectionCalculator(self)
        self.GENFIRE_ProjectionCalculator.model_loading_signal.connect(partial(self.receive_msg, "Loading Model..."))
        self.GENFIRE_ProjectionCalculator.emit_message_signal.connect(self.receive_msg)
        self.GENFIRE_ProjectionCalculator.update_filenames_signal.connect(self.updateFilenames)
        self.GENFIRE_ProjectionCalculator.show()

    def setGriddingMethod(self):
        if self.ui.radioButton_DFT.isChecked():
            self.GENFIRE_ReconstructionParameters.griddingMethod = "DFT"
        else:
            self.GENFIRE_ReconstructionParameters.griddingMethod = "FFT"

    def updateFilenames(self):
        import os
        base, file = os.path.split(toString(self.GENFIRE_ProjectionCalculator.calculationParameters.outputFilename))

        self.ui.lineEdit_angle.setText(toQString(str(self.GENFIRE_ProjectionCalculator.calculationParameters.outputAngleFilename)))
        self.ui.lineEdit_angle.textChanged.emit(self.ui.lineEdit_angle.text())
        self.ui.lineEdit_pj.setText(toQString(str(self.GENFIRE_ProjectionCalculator.calculationParameters.outputFilename)))
        self.ui.lineEdit_pj.textChanged.emit(self.ui.lineEdit_pj.text())
        self.ui.lineEdit_results.setText(toQString(str(base + "/results.mrc")))
        self.ui.lineEdit_results.textChanged.emit(self.ui.lineEdit_results.text())

    def launchVolumeSlicer(self):
        import genfire.fileio
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select Volume",filter="Volume files (*.mat *.mrc *.npy);;All Files (*)")
        if filename:
            filename = toString(filename)
            volume = genfire.fileio.readVolume(filename)
            self.VolumeSlicer = VolumeSlicer.VolumeSlicer(volume)
            self.VolumeSlicer.show()

    def toggleResCircle(self):
        if self.ui.checkBox_resCircle.isChecked():
            self.GENFIRE_ReconstructionParameters.enforceResolutionCircle = True
        else:
            self.GENFIRE_ReconstructionParameters.enforceResolutionCircle = False

    def toggleMultiGridding(self):
        if self.ui.checkBox_multiGridding.isChecked():
            self.GENFIRE_ReconstructionParameters.permitMultipleGridding = True
        else:
            self.GENFIRE_ReconstructionParameters.permitMultipleGridding = False

    def toggleSelectIO(self):
        if self.ui.lineEdit_io.isEnabled():
             self.ui.lineEdit_io.setStyleSheet("background-color: gray")
             self.ui.lineEdit_io.setEnabled(False)
             self.GENFIRE_ReconstructionParameters.initialObjectFilename= ""
        else:
             self.ui.lineEdit_io.setStyleSheet("background-color: white")
             self.ui.lineEdit_io.setEnabled(True)

    def toggleSupportConstraint(self):
        if self.ui.checkBox_support_constraint.isChecked():
            self.GENFIRE_ReconstructionParameters.constraint_support = True
        else:
            self.GENFIRE_ReconstructionParameters.constraint_support = False

    def togglePositivityConstraint(self):
        if self.ui.checkBox_positivity_constraint.isChecked():
            self.GENFIRE_ReconstructionParameters.constraint_positivity = True
        else:
            self.GENFIRE_ReconstructionParameters.constraint_positivity = False

    def toggleUseDefaultSupport(self):
        if self.ui.checkBox_default_support.isChecked():
            self.GENFIRE_ReconstructionParameters.useDefaultSupport = True
            self.ui.lineEdit_support.setDisabled(True)
            self.ui.lineEdit_support.setStyleSheet("background-color: gray")
        else:
            self.GENFIRE_ReconstructionParameters.useDefaultSupport = False
            self.ui.lineEdit_support.setDisabled(False)
            self.ui.lineEdit_support.setStyleSheet("background-color: white")

    def messageWritten(self, message):
        print(message)

    def __del__(self):
        import sys
        sys.stdout = sys.__stdout__ # Restore output stream to default upon exiting the GUI
        sys.stderr = sys.__stderr__

    #Functions for selecting input files using QFileDialog
    def selectProjectionFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select File Containing Projections",filter="Projection Stacks (*.mrc *.mat *.tif *.npy);; MATLAB files (*.mat);;TIFF images (*.tif *.tiff);;MRC (*.mrc);;All Files (*)")

        if filename:
            self.GENFIRE_ReconstructionParameters.setProjectionFilename(filename)
            self.ui.lineEdit_pj.setText(toQString(filename))

    def selectAngleFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select File Containing Support",filter="Euler Angles (*.txt *.mat);; MATLAB files (*.mat);;text files (*.txt);;All Files (*)")
        if filename:
            self.GENFIRE_ReconstructionParameters.setAngleFilename(filename)
            self.ui.lineEdit_angle.setText(toQString(filename))

    def selectSupportFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select File Containing Support", filter="Volume Files (*.mrc *.mat *.npy);; MATLAB files (*.mat);;MRC (*.mrc);;All Files (*)")
        if filename:
            self.GENFIRE_ReconstructionParameters.setSupportFilename(filename)
            self.ui.lineEdit_support.setText(toQString(filename))
            self.ui.checkBox_default_support.setChecked(False)

    def selectInitialObjectFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select File Containing Initial Object", filter="Volume Files (*.mrc *.mat *.npy);; MATLAB files (*.mat);;MRC (*.mrc);;All Files (*)")
        if filename:
            self.GENFIRE_ReconstructionParameters.setInitialObjectFilename(filename)
            self.ui.lineEdit_io.setText(toQString(filename))

    #Define constraint enforcement mode
    def selectResolutionExtensionSuppressionState(self):
        if self.ui.radioButton_on.isChecked():
            self.GENFIRE_ReconstructionParameters.setResolutionExtensionSuppressionState(1)
        elif self.ui.radioButton_extension.isChecked():
            self.GENFIRE_ReconstructionParameters.setResolutionExtensionSuppressionState(3)
        else:
            self.GENFIRE_ReconstructionParameters.setResolutionExtensionSuppressionState(2)

    def checkParameters(self):
        parametersAreGood = self.GENFIRE_ReconstructionParameters.checkParameters()
        if parametersAreGood: #update "go" button if we are ready
            self.ui.btn_reconstruct.setEnabled(True)
            self.ui.btn_reconstruct.setStyleSheet("background-color: GREEN; color:#ffffff;font-size:30px")
            # GF_window.ui.btn_reconstruct.setStyleSheet("color: WHITE")
            self.ui.btn_reconstruct.setText("Launch Reconstruction!")

    #Function to run GENFIRE reconstruction once all parameters are accounted for

    @QtCore.pyqtSlot()
    def startReconstruction(self):
        print('Launching GENFIRE reconstruction')
        # Launch the reconstruction in a separate thread to prevent the GUI blocking while reconstructing
        from threading import Thread
        from functools import partial
        import genfire.main
        t = Thread(target=partial(genfire.main.main, self.GENFIRE_ReconstructionParameters))
        t.start()
    def displayResults(self):
        outputfilename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select Reconstruction",filter="Volume files (*.mrc *.mat *.npy)  ;; MATLAB files (*.mat);;text files (*.txt *.tiff);;MRC (*.mrc);;All Files (*)")
        outputfilename = toString(outputfilename)
        if outputfilename:
            import numpy as np
            import os
            import genfire.fileio
            import matplotlib.pyplot as plt
            plt.close('all')

            initialObject = genfire.fileio.readVolume(outputfilename)
            dims = np.shape(initialObject)
            n_half_x = int(dims[0]/2) #this assumes even-sized arrays
            n_half_y = int(dims[1]/2)
            n_half_z = int(dims[2]/2)
            reconstructionDisplayWindowSize=dims[0] # array should be cubic
            half_window_x = reconstructionDisplayWindowSize//2
            half_window_y = reconstructionDisplayWindowSize//2
            half_window_z = reconstructionDisplayWindowSize//2
            plt.figure()
            plt.subplot(233)
            plt.imshow(np.squeeze(initialObject[n_half_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z]))
            plt.title("central YZ slice")

            plt.subplot(232)
            plt.imshow(np.squeeze(initialObject[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y, n_half_z-half_window_z:n_half_z+half_window_z]))
            plt.title("central XZ slice")

            plt.subplot(231)
            plt.title("central XY slice")
            plt.imshow(np.squeeze(initialObject[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z]))

            plt.subplot(236)
            plt.title("YZ projection")
            plt.imshow(np.squeeze(np.sum(initialObject[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z], axis=0)))

            plt.subplot(235)
            plt.title("XZ projection")
            plt.imshow(np.squeeze(np.sum(initialObject[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z], axis=1)))

            plt.subplot(234)
            plt.title("XY projection")
            plt.imshow(np.squeeze(np.sum(initialObject[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z], axis=2)))
            plt.get_current_fig_manager().window.setGeometry(25,25,600, 1200)

            # now load error curves if they exist
            output_base, output_ext = os.path.splitext(outputfilename)
            errK_filename = output_base + '_errK.txt'
            if os.path.isfile(errK_filename):
                errK = np.loadtxt(errK_filename)
                numIterations = np.shape(errK)[0]
                plt.figure()
                plt.plot(range(0,numIterations),errK)

                mngr = plt.get_current_fig_manager()
                mngr.window.setGeometry(700,25,550, 250)
                plt.title("Reciprocal Error vs Iteration Number")
                plt.xlabel("Iteration Num")
                plt.ylabel("Reciprocal Error")
                # plt.setp(plt.gcf(),)
            Rfree_total_filename = output_base + '_Rfree_total.txt'
            Rfree_bybin_filename = output_base + '_Rfree_bybin.txt'
            if os.path.isfile(Rfree_total_filename):
                Rfree_total = np.loadtxt(Rfree_total_filename)
                numIterations = np.shape(Rfree_total)[0]
                plt.figure()
                plt.title("R-free vs Iteration Number")
                mngr = plt.get_current_fig_manager()
                mngr.window.setGeometry(700,350,550, 250)
                plt.plot(range(0,numIterations),Rfree_total)
                plt.title("Mean R-free Value vs Iteration Number")
                plt.xlabel("Iteration Num")
                plt.ylabel('Mean R-free')

            if os.path.isfile(Rfree_bybin_filename):
                Rfree_bybin = np.loadtxt(Rfree_bybin_filename)
                plt.figure()
                mngr = plt.get_current_fig_manager()
                mngr.window.setGeometry(700,650,550, 250)
                X = np.linspace(0,1,np.shape(Rfree_bybin)[0])
                plt.plot(X, Rfree_bybin[:,-1])
                plt.title("Final Rfree Value vs Spatial Frequency")
                plt.xlabel("Spatial Frequency (% of Nyquist)")
                plt.ylabel('Rfree')

            plt.draw()
            plt.pause(1e-30) # slight pause forces redraw

    @QtCore.pyqtSlot(str)
    def receive_msg(self, msg):
        global process_finished
        if not process_finished:
            self.ui.log.moveCursor(QtGui.QTextCursor.End)
            formatted_msg = "<span style=\" font-size:14pt; font-weight:600; color:#000000;\" >" + msg + "</span/>"
            self.ui.log.append(formatted_msg)

    @QtCore.pyqtSlot(str)
    def receive_error_msg(self, msg):
        global process_finished
        if not process_finished:
            self.ui.log.moveCursor(QtGui.QTextCursor.End)
            formatted_msg = "<span style=\" font-size:14pt; font-weight:600; color:#ff0000;\" >" + msg + "</span/>"
            self.ui.log.append(formatted_msg)

    @QtCore.pyqtSlot()
    def stopRunning(self):
        global process_finished
        process_finished = True

class GenfireListener(QtCore.QObject):
    message_pending = QtCore.pyqtSignal(str)
    def __init__(self, msg_queue):
        super(GenfireListener, self).__init__()
        self.msg_queue = msg_queue
        self.process_finished = False

    @QtCore.pyqtSlot()
    def run(self):
        msg = ''
        while not self.process_finished:
            msg = self.msg_queue.get() #get next message, blocks if nothing to get
            if self.process_finished:
                self.message_pending.emit(msg)
                return
            self.message_pending.emit(msg)

    @QtCore.pyqtSlot()
    def stopRunning(self):
        self.process_finished = True

class GenfireWriter(object):
    def __init__(self, msg_queue):
        self.msg_queue = msg_queue

    def write(self, message):
        if message != "\n":
            self.msg_queue.put("GENFIRE: " + message)

class GenfireLogger(QtCore.QObject):
    def __init__(self, msg_queue):
        super(GenfireLogger, self).__init__()
        self.msg_queue = msg_queue
        self.listener  = GenfireListener(msg_queue=self.msg_queue)
        self.listener_thread = QtCore.QThread()
        self.listener.moveToThread(self.listener_thread)
        self.listener_thread.started.connect(self.listener.run)
        self.listener_thread.start()

    @QtCore.pyqtSlot()
    def cleanup_thread(self):
        global process_finished
        self.listener.process_finished = True
        self.msg_queue.put("Safely Exit.") # write a final message to force i/o threads to unblock and see the exit flag
        if self.listener_thread.isRunning():
            self.listener_thread.quit()
            self.listener_thread.wait()

def main():

    # Startup the application
    app = QtWidgets.QApplication(sys.argv)
    # app.setStyle('plastique')
    app.setStyle('mac')

    # Create the GUI
    GF_window  = GenfireMainWindow()

    # Render GUI
    GF_window.show()

    # Redirect standard output to the GUI
    if sys.version_info >= (3,0):
        from multiprocessing import Queue
    else:
        from Queue import Queue
    global process_finished
    global GF_logger
    global GF_error_logger
    process_finished = False # flag to control save exit of the i/o threads for logging

    msg_queue  = Queue()
    GF_logger  = GenfireLogger(msg_queue)
    sys.stdout = GenfireWriter(msg_queue)
    GF_logger.listener.message_pending[str].connect(GF_window.receive_msg)

    err_msg_queue = Queue()
    GF_error_logger  = GenfireLogger(err_msg_queue)
    sys.stderr = GenfireWriter(err_msg_queue)
    GF_error_logger.listener.message_pending[str].connect(GF_window.receive_error_msg)

    GF_window.stop_threads.connect(GF_error_logger.cleanup_thread)
    GF_window.stop_threads.connect(GF_logger.cleanup_thread)

    GF_window.stop_threads.connect(GF_error_logger.listener.stopRunning)
    GF_window.stop_threads.connect(GF_logger.listener.stopRunning)

    # Start event loop
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()





"""
* genfire.reconstruct *

This module contains the core of GENFIRE's functions for computing reconstructions


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""


from __future__ import division
import matplotlib
matplotlib.use("Qt5Agg")
import matplotlib.pyplot as plt
import os
import scipy.io
import time
import genfire
from multiprocessing import Pool
from genfire.utility import *

PI = np.pi
if __name__ != "__main__":
    def reconstruct(numIterations, initialObject, support, measuredK, constraintIndicators, constraintEnforcementDelayIndicators, R_freeInd_complex, R_freeVals_complex, displayFigure, use_positivity=True, use_support=True):
        """
         * reconstruct *

         Primary GENFIRE reconstruction function

        :param numIterations: Integer number of iterations to run
        :param initialObject: Initial guess of 3D object
        :param support: Binary matrix representing where the object is allowed to exist
        :param measuredK: Assembled 3D Fourier grid
        :param constraintIndicators: Flag value for each datapoint used to control during which iterations a given Fourier component is enforced
        :param constraintEnforcementDelayIndicators: List of cutoff values that will be divided evenly among the iterations. All Fourier grid points with constraintIndicators greater than the current cutoff are enforced
        :param R_freeInd_complex: 3 x 1 x num_shells tuple of (x,y,z) coordinate lists for withheld values for Rfree calculation
        :param R_freeVals_complex: Complex valued component at the indices given by R_freeInd_complex
        :param displayFigure: Boolean flag to display figures during the reconstruction
        :return: outputs dictionary containing the reconstruction and error metrics

         Author: Alan (AJ) Pryor, Jr.
         Jianwei (John) Miao Coherent Imaging Group
         University of California, Los Angeles
         Copyright 2015-2016. All rights reserved.
        """
        import time
        t0 = time.time()
        print("Reconstruction started")
        bestErr = 1e30 #initialize error

        #initialize arrays for error metrics
        Rfree_complex = np.ones(numIterations)*-1 #
        errK = np.zeros(numIterations)

        #prefetch indices for monitoring error
        errInd = measuredK != 0

        #get dimensions of object
        dims = np.shape(support)
        if R_freeInd_complex:
            Rfree_complex_bybin = np.zeros((np.shape(R_freeInd_complex)[2], numIterations),dtype=float)
            Rfree_complex_total = np.zeros(numIterations,dtype=float)

        if displayFigure.DisplayFigureON: #setup some indices for plotting.
            n_half_x = int(dims[0]/2) #this assumes even-sized arrays
            n_half_y = int(dims[1]/2)
            n_half_z = int(dims[2]/2)

            half_window_x = displayFigure.reconstructionDisplayWindowSize[0]//2
            half_window_y = displayFigure.reconstructionDisplayWindowSize[1]//2
            half_window_z = displayFigure.reconstructionDisplayWindowSize[2]//2

        #setup output dict
        if R_freeInd_complex:
            outputs = {'reconstruction':initialObject,'errK':errK,'R_free_bybin':Rfree_complex_bybin, "R_free_total":Rfree_complex_total}
        else:
            outputs = {'reconstruction':initialObject,'errK':errK}

        #determine how to divide up the constraint enforcement cutoffs among the iterations by determining
        #which iterations will require a recalculation of the indices to enforce
        iterationNumsToChangeCutoff = np.round(np.linspace(1, numIterations, num=np.size(constraintEnforcementDelayIndicators)))
        iterationNumsToChangeCutoff, uniqueIndices = np.unique(iterationNumsToChangeCutoff, return_index=True)
        iterationNumsToChangeCutoff = np.append(iterationNumsToChangeCutoff,1e30) #add an arbitrarily high number to the end that is an iteration number that won't be reached
        constraintEnforcementDelayIndicators = constraintEnforcementDelayIndicators[uniqueIndices]
        currentCutoffNum = 0
        for iterationNum in range(1, numIterations+1): #iterations are counted started from 1

            if iterationNum == iterationNumsToChangeCutoff[currentCutoffNum]: #update current Fourier constraint if appropriate
                relevantCutoff = constraintEnforcementDelayIndicators[currentCutoffNum]
                constraintInd_complex = (constraintIndicators > relevantCutoff) * measuredK != 0

                bestErr = 1e30 #reset error
                currentCutoffNum+=1#update constraint set number

            if use_positivity:
                initialObject[initialObject<0] = 0 #enforce positivity
            if use_support:
                initialObject = initialObject * support #enforce support

            #take FFT of current reconstruction
            k = rfftn(initialObject)

            #compute error
            errK[iterationNum-1] = np.sum(abs(np.abs(k[errInd])-np.abs(measuredK[errInd])))/np.sum(abs(measuredK[errInd]))#monitor error
            print("Iteration number: {0}           error = {1:0.5f}".format(iterationNum, errK[iterationNum-1]))

            #update best object if a better one has been found
            if errK[iterationNum-1] < bestErr:
                bestErr = errK[iterationNum-1]
                outputs['reconstruction'] = initialObject

            #calculate Rfree for each spatial frequency shell if necessary
            if R_freeInd_complex:
                total_Rfree_error      = 0
                total_Rfree_error_norm = 0
                for shellNum in range(0, np.shape(R_freeInd_complex)[2]):

                    tmpIndX = R_freeInd_complex[0][0][shellNum]
                    tmpIndY = R_freeInd_complex[1][0][shellNum]
                    tmpIndZ = R_freeInd_complex[2][0][shellNum]

                    tmpVals = np.abs(R_freeVals_complex[shellNum])
                    # Rfree_numerator                         = np.sum(abs(k[tmpIndX, tmpIndY, tmpIndZ] - tmpVals))
                    Rfree_numerator                         = np.sum(abs(np.abs(k[tmpIndX, tmpIndY, tmpIndZ]) - tmpVals))
                    Rfree_denominator                       = np.sum(abs(tmpVals))
                    total_Rfree_error                      += Rfree_numerator
                    total_Rfree_error_norm                 += Rfree_denominator
                    Rfree_complex_bybin[shellNum, iterationNum-1] = Rfree_numerator / Rfree_denominator
                Rfree_complex_total[iterationNum-1] = total_Rfree_error / total_Rfree_error_norm
            #replace Fourier components with ones from measured data from the current set of constraints
            k[constraintInd_complex] = measuredK[constraintInd_complex]
            initialObject = irfftn(k)

            #update display
            if displayFigure.DisplayFigureON:
                if iterationNum % displayFigure.displayFrequency == 0:
                    print("n_half_x = ", n_half_x)
                    print("half_window_y = ", half_window_y)
                    plt.figure(1000)
                    plt.subplot(233)
                    plt.imshow(np.squeeze(np.fft.fftshift(initialObject)[n_half_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z]))
                    plt.title("central YZ slice")

                    plt.subplot(232)
                    plt.imshow(np.squeeze(np.fft.fftshift(initialObject)[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y, n_half_z-half_window_z:n_half_z+half_window_z]))
                    plt.title("central XZ slice")

                    plt.subplot(231)
                    plt.title("central XY slice")
                    plt.imshow(np.squeeze(np.fft.fftshift(initialObject)[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z]))

                    plt.subplot(236)
                    plt.title("YZ projection")
                    plt.imshow(np.squeeze(np.sum(np.fft.fftshift(initialObject)[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z], axis=0)))

                    plt.subplot(235)
                    plt.title("XZ projection")
                    plt.imshow(np.squeeze(np.sum(np.fft.fftshift(initialObject)[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z], axis=1)))

                    plt.subplot(234)
                    plt.title("XY projection")
                    plt.imshow(np.squeeze(np.sum(np.fft.fftshift(initialObject)[n_half_x-half_window_x:n_half_x+half_window_x, n_half_y-half_window_y:n_half_y+half_window_y, n_half_z-half_window_z:n_half_z+half_window_z], axis=2)))
                    plt.get_current_fig_manager().window.setGeometry(25,25,400, 400)
                    plt.draw()

                    plt.figure(2)
                    plt.get_current_fig_manager().window.setGeometry(25,450,400, 400)
                    plt.plot(range(0,numIterations),errK)
                    plt.title("K-space Error vs Iteration Number")
                    plt.xlabel("Spatial Frequency (% of Nyquist)")
                    plt.ylabel('Reciprocal Space Error')
                    plt.draw()

                    if R_freeInd_complex:
                        plt.figure(3)
                        mngr = plt.get_current_fig_manager()
                        mngr.window.setGeometry(450,25,400, 400)
                        plt.plot(range(0,numIterations),Rfree_complex_total)
                        plt.title("Mean R-free Value vs Iteration Number")
                        plt.xlabel("Iteration Num")
                        plt.ylabel('Mean R-free')
                        plt.draw()

                        plt.figure(4)
                        mngr = plt.get_current_fig_manager()
                        mngr.window.setGeometry(450,450,400, 400)
                        X = np.linspace(0,1,np.shape(Rfree_complex_bybin)[0])
                        plt.plot(X, Rfree_complex_bybin[:,iterationNum-1])
                        plt.title("Current Rfree Value vs Spatial Frequency")
                        plt.xlabel("Spatial Frequency (% of Nyquist)")
                        plt.ylabel('Rfree')
                        plt.draw()


                    plt.pause(1e-30) #forces display to update

        outputs['errK'] = errK
        if R_freeInd_complex:
            outputs['R_free_bybin'] = Rfree_complex_bybin
            outputs['R_free_total'] = Rfree_complex_total
        outputs['reconstruction'] = np.fft.fftshift(outputs['reconstruction'])
        print("Reconstruction finished in {0:0.1f} seconds".format(time.time()-t0))
        return outputs


    def fillInFourierGrid(projections,angles,interpolationCutoffDistance, enforce_resolution_circle=True, permitMultipleGridding=True):
        """
        * fillInFourierGrid *

        FFT gridding function for converting a set of 2D projection images into a 3D Fourier grid

        :param projections: N x N x num_projections NumPy array containing the projections
        :param angles: 3 x num_projections NumPy array of Euler angles phi,theta, psi
        :param interpolationCutoffDistance: Radius of interpolation kernel. Only values within this radius of a grid point are considered
        :param enforce_resolution_circle: boolean; whether or not to truncate reciprocal space to Nyquist frequency
        :return: the assembled Fourier grid

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.

        """
        print ("Assembling Fourier grid.")
        tic = time.time()
        dim1 = np.shape(projections)[0]
        dim2 = np.shape(projections)[1]
        if len(np.shape(projections))>2:
            numProjections = np.shape(projections)[2]
        else:
            numProjections = 1
        nc = np.round(dim1/2)
        n2 = nc
        measuredX = np.zeros([dim1*dim2,numProjections])
        measuredY = np.zeros([dim1*dim2,numProjections])
        measuredZ = np.zeros([dim1*dim2,numProjections])
        kMeasured = np.zeros([dim1,dim1,numProjections], dtype=complex)
        # confidenceWeights = np.zeros([dim1,dim1,numProjections])
        ky,kx = np.meshgrid(np.arange(-n2,n2,1,dtype=float),np.arange(-n2,n2,1,dtype=float))
        Q = np.sqrt(ky**2+kx**2)/n2
        kx = np.reshape(kx, [1, dim1*dim2], 'F')
        ky = np.reshape(ky, [1, dim1*dim2], 'F')
        kz = np.zeros([1, dim1*dim1])
        for projNum in range(0, numProjections):

            # convert angles to radians and construct the rotation matrix
            phi = angles[projNum, 0] * PI/180
            theta = angles[projNum, 1] * PI/180
            psi = angles[projNum, 2] * PI/180
            R = np.array([[np.cos(psi)*np.cos(theta)*np.cos(phi)-np.sin(psi)*np.sin(phi) ,np.cos(psi)*np.cos(theta)*np.sin(phi)+np.sin(psi)*np.cos(phi)   ,    -np.cos(psi)*np.sin(theta)],
            [-np.sin(psi)*np.cos(theta)*np.cos(phi)-np.cos(psi)*np.sin(phi), -np.sin(psi)*np.cos(theta)*np.sin(phi)+np.cos(psi)*np.cos(phi) ,   np.sin(psi)*np.sin(theta) ],
            [np.sin(theta)*np.cos(phi)                               , np.sin(theta)*np.sin(phi)                                ,              np.cos(theta)]])
            R = R.T

            Kcoordinates = np.zeros([3, dim1*dim2],dtype=float)
            Kcoordinates[0, :] = kx
            Kcoordinates[1, :] = ky
            Kcoordinates[2, :] = kz


            rotkCoords = np.dot(R, Kcoordinates)
            measuredX[:, projNum] = rotkCoords[0, :]
            measuredY[:, projNum] = rotkCoords[1, :]
            measuredZ[:, projNum] = rotkCoords[2, :]
            kMeasured[:, :, projNum] = fftn_fftshift(projections[:, :, projNum])

        # reorganize the coordinates and discard any flagged values
        measuredX = np.reshape(measuredX,[1, np.size(kMeasured)], 'F')
        measuredY = np.reshape(measuredY,[1, np.size(kMeasured)], 'F')
        measuredZ = np.reshape(measuredZ,[1, np.size(kMeasured)], 'F')
        kMeasured = np.reshape(kMeasured,[1, np.size(kMeasured)], 'F')
        notFlaggedIndices = kMeasured != -999
        measuredX = measuredX[notFlaggedIndices]
        measuredY = measuredY[notFlaggedIndices]
        measuredZ = measuredZ[notFlaggedIndices]
        kMeasured = kMeasured[notFlaggedIndices]

        masterInd = []
        masterVals = []
        masterDistances = []

        # check whether we need to consider multiple grid points
        if permitMultipleGridding:
            shiftMax = int(round(interpolationCutoffDistance))
        else:
            shiftMax = 0
        for Yshift in range(-shiftMax, shiftMax+1):
            for Xshift in range(-shiftMax, shiftMax+1):
                for Zshift in range(-shiftMax, shiftMax+1):

                    tmpX = np.round(measuredX) + Xshift
                    tmpY = np.round(measuredY) + Yshift
                    tmpZ = np.round(measuredZ) + Zshift

                    tmpVals = kMeasured
                    # tmpConfidenceWeights = confidenceWeights
                    distances = np.abs(measuredX-tmpX)**2 + np.abs(measuredY-tmpY)**2 + np.abs(measuredZ-tmpZ)**2
                    tmpX+=nc
                    tmpY+=nc
                    tmpZ+=nc

                    goodInd = (np.logical_not((tmpX > (dim1-1)) | (tmpX < 0) | (tmpY > (dim1-1)) | (tmpY < 0) | (tmpZ > (dim1-1)) | (tmpZ < 0))) & (distances <= (interpolationCutoffDistance**2))

                    masterInd=np.append(masterInd, np.ravel_multi_index((tmpX[goodInd].astype(np.int64), tmpY[goodInd].astype(np.int64), tmpZ[goodInd].astype(np.int64)),[dim1, dim1, dim1], order='F'))
                    masterVals=np.append(masterVals, tmpVals[goodInd])
                    masterDistances=np.append(masterDistances, distances[goodInd])


        masterInd = np.array(masterInd).astype(np.int64)
        masterVals = np.array(masterVals)
        masterDistances = np.array(masterDistances)

        #  only assemble half of the grid and then fill the remainder by Hermitian symmetry
        halfwayCutoff = ((dim1+1)**3)//2+1
        masterVals = masterVals[masterInd <= halfwayCutoff]
        masterDistances = masterDistances[masterInd <= halfwayCutoff]
        masterDistances = masterDistances +  1e-5
        masterDistances [masterDistances != 0 ]  = 1 / masterDistances[masterDistances != 0 ]
        masterInd = masterInd[masterInd <= halfwayCutoff]

        measuredK = np.zeros([dim1**3], dtype=complex)

        # accumulate the sums
        vals_real = np.bincount(masterInd, weights=(masterDistances * np.real(masterVals)))
        vals_cx = np.bincount(masterInd, weights=(masterDistances * np.imag(masterVals)))
        vals = vals_real + 1j * vals_cx
        sum_weights = np.bincount(masterInd, weights=(masterDistances))
        vals[sum_weights != 0] = vals[sum_weights != 0] / sum_weights[sum_weights != 0]
        measuredK[np.arange(np.size(vals))] = vals
        measuredK = np.reshape(measuredK,[dim1,dim1,dim1],order='F')

        measuredK[np.isnan(measuredK)] = 0

        if enforce_resolution_circle:
            Q = genfire.utility.generateKspaceIndices(measuredK)
            measuredK[Q>1] = 0

        # apply Hermitian symmetry
        measuredK = genfire.utility.hermitianSymmetrize(measuredK)

        print ("Fourier grid assembled in {0:0.1f} seconds".format(time.time()-tic))
        return measuredK



    def fillInFourierGrid_DFT(projections,angles,interpolationCutoffDistance, enforce_resolution_circle):
        """
        * fillInFourierGrid_DFT *

        DFT gridding function for converting a set of 2D projection images into a 3D Fourier grid

        :param projections: N x N x num_projections NumPy array containing the projections
        :param angles: 3 x num_projections NumPy array of Euler angles phi,theta, psi
        :param interpolationCutoffDistance: Radius of interpolation kernel. Only values within this radius of a grid point are considered
        :param enforce_resolution_circle: boolean; whether or not to truncate reciprocal space to Nyquist frequency
        :return: the assembled Fourier grid


        Author: Yongsoo Yang
        Transcribed from MATLAB codes by Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.
        """


        print ("Assembling Fourier grid.")
        tic = time.time()
        from genfire.utility import pointToPlaneClosest, pointToPlaneDistance
        (n1, n2) = (np.shape(projections)[0],np.shape(projections)[1])
        minInvThresh = 0.00001
        num_projections = np.shape(projections)[2]
        normVECs = np.zeros((num_projections,3))
        rotMATs  = np.zeros((3,3,num_projections))
        phis     = angles[:, 0] * PI/180
        thetas   = angles[:, 1] * PI/180
        psis     = angles[:, 2] * PI/180
        init_normvec = np.array([0, 0, 1],dtype=float)
        for ang_num in range(num_projections):
            phi   = phis[ang_num]
            theta = thetas[ang_num]
            psi   = psis[ang_num]
            R     = np.array([[np.cos(psi)*np.cos(theta)*np.cos(phi)-np.sin(psi)*np.sin(phi) ,np.cos(psi)*np.cos(theta)*np.sin(phi)+np.sin(psi)*np.cos(phi)   ,    -np.cos(psi)*np.sin(theta)],
            [-np.sin(psi)*np.cos(theta)*np.cos(phi)-np.cos(psi)*np.sin(phi), -np.sin(psi)*np.cos(theta)*np.sin(phi)+np.cos(psi)*np.cos(phi) ,   np.sin(psi)*np.sin(theta) ],
            [np.sin(theta)*np.cos(phi)                               , np.sin(theta)*np.sin(phi)                                ,              np.cos(theta)]])
            rotMATs[:, :, ang_num] = R.T
            normVECs[ang_num, :] = np.dot(R.T, init_normvec.T)
        k1               = np.arange(-1*(n1//2), 1, 1, dtype=float)
        k1_full          = np.arange(-1*(n1//2), n1//2 + 1, 1, dtype=float)
        k2               = np.arange(-1 * n2//2, n2//2 + 1, 1, dtype=float)
        k3               = np.arange(-1 * n1//2, n1//2 + 1, 1, dtype=float)
        (null, K1, null) = np.meshgrid(k2, k1, k3)
        FS               = np.zeros_like(K1, dtype=complex)
        Numpt            = np.zeros_like(K1)
        invSumTotWeight  = np.zeros_like(K1, dtype=float)
        [K20, K10] = np.meshgrid(k2[:-1],k1_full[:-1])
        K20 = K20.flatten()
        K10 = K10.flatten()
        try:
            artifical_error = 1/0 # This loop uses broadcasting, which can use more memory, so there is a try/catch as a failsafe. However
            # the simple loop seems to be faster, so for the moment I have just introduced an error artificially as a lazy way of leaving both implementations
            for proj_num in range(num_projections):
                curr_proj = projections[:, :, proj_num].flatten()
                [K2, K1, K3] = np.meshgrid(k2,k1,k3)
                D = pointToPlaneDistance(np.vstack((K1.flatten(order="F"), K2.flatten(order="F"), K3.flatten(order="F"))).T, normVECs[proj_num,:])
                Dind = np.array(np.where(D < interpolationCutoffDistance))
                CP = pointToPlaneClosest(np.vstack((K1.flatten(order="F")[[Dind]], K2.flatten(order="F")[[Dind]], K3.flatten(order="F")[[Dind]])).T, normVECs[proj_num,:].T,np.zeros(np.size(Dind),dtype=float))
                CP_plane = np.dot(np.linalg.inv(rotMATs[:, :, proj_num]), CP.T)

                Gind = Dind[0,(abs(CP_plane[0,:]) <= n1/2) & (abs(CP_plane[1,:]) <= n2/2)]
                G_CP_plane = CP_plane[:,( abs(CP_plane[0,:]) <= n1/2 )& (abs(CP_plane[1,:]) <= n2/2) ]
                nonzero_ind = np.where(curr_proj!=0)
                X = np.zeros((np.shape(G_CP_plane)[1], 1), dtype=float)
                Y = np.zeros((np.shape(G_CP_plane)[1], 1), dtype=float)
                X[:, 0] = G_CP_plane[0, :]
                Y[:, 0] = G_CP_plane[1, :]

                Fpoints = np.sum( curr_proj[nonzero_ind] * np.exp( -1j*2*PI * (K10[nonzero_ind] * X / n1 + K20[nonzero_ind] * Y / n2  ) ) ,axis=1)
                distances = D[Gind]
                distances[ distances < minInvThresh ] = minInvThresh
                currTotWeight = 1 / distances
                Gind = np.unravel_index(Gind, np.shape(FS), order="F")
                FS[Gind] = FS[Gind] * invSumTotWeight[Gind] + currTotWeight * Fpoints
                invSumTotWeight[Gind] = invSumTotWeight[Gind] + currTotWeight
                FS[Gind] = FS[Gind] / invSumTotWeight[Gind]
                Numpt[Gind] = Numpt[Gind] + 1
        except:
            for proj_num in range(num_projections):
                curr_proj = projections[:, :, proj_num].flatten()
                [K2, K1, K3] = np.meshgrid(k2,k1,k3)
                D = pointToPlaneDistance(np.vstack((K1.flatten(order="F"), K2.flatten(order="F"), K3.flatten(order="F"))).T, normVECs[proj_num,:])
                Dind = np.array(np.where(D < interpolationCutoffDistance))
                CP = pointToPlaneClosest(np.vstack((K1.flatten(order="F")[[Dind]], K2.flatten(order="F")[[Dind]], K3.flatten(order="F")[[Dind]])).T, normVECs[proj_num,:].T,np.zeros(np.size(Dind),dtype=float))
                CP_plane = np.dot(np.linalg.inv(rotMATs[:, :, proj_num]), CP.T)

                Gind = Dind[0,(abs(CP_plane[0,:]) <= n1/2) & (abs(CP_plane[1,:]) <= n2/2)]
                G_CP_plane = CP_plane[:,( abs(CP_plane[0,:]) <= n1/2 )& (abs(CP_plane[1,:]) <= n2/2) ]
                Fpoints = np.zeros(np.shape(G_CP_plane)[1], dtype=complex)

                nonzero_ind = np.where(curr_proj!=0)
                # for i,v in enumerate(G_CP_plane.T):
                #     Fpoints[i] = np.sum( curr_proj[nonzero_ind] * np.exp( -1j*2*PI * (K10[nonzero_ind] * v[0] / n1 + K20[nonzero_ind] * v[1] / n2  ) ) )


                K10_t = K10[nonzero_ind]
                K20_t = K20[nonzero_ind]
                curr_proj = curr_proj[nonzero_ind]

                for i in range(np.shape(G_CP_plane)[1]):
                    Fpoints[i] = np.sum( curr_proj * np.exp( -1j*2*PI * (K10_t * G_CP_plane[0, i] / n1 + K20_t * G_CP_plane[1, i] / n2  ) ) )
                    # Fpoints[i] = np.sum( curr_proj[nonzero_ind] * np.exp( -1j*2*PI * (K10[nonzero_ind] * G_CP_plane[0, i] / n1 + K20[nonzero_ind] * G_CP_plane[1, i] / n2  ) ) )
                distances = D[Gind]
                distances[ distances < minInvThresh ] = minInvThresh
                currTotWeight = 1 / distances
                Gind = np.unravel_index(Gind, np.shape(FS), order="F")
                FS[Gind] = FS[Gind] * invSumTotWeight[Gind] + currTotWeight * Fpoints
                invSumTotWeight[Gind] = invSumTotWeight[Gind] + currTotWeight
                FS[Gind] = FS[Gind] / invSumTotWeight[Gind]
                Numpt[Gind] = Numpt[Gind] + 1
        # measuredK = np.zeros((n1,n2,n1), dtype=complex)
        measuredK = np.zeros((n1+1,n2+1,n1+1), dtype=complex)
        measuredK[:n1//2 + 1, :, :] = FS[:, :, :]
        print ("Fourier grid assembled in {0:0.1f} seconds".format(time.time()-tic))

        if enforce_resolution_circle:
            Q = genfire.utility.generateKspaceIndices(measuredK)
            measuredK[Q>1] = 0

        return genfire.utility.hermitianSymmetrize(measuredK)[:-1,:-1,:-1]

    def readMAT(filename):
        """
        * readMAT *

        Read projections from a .mat file

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.

        :param filename: MATLAB file (.mat) containing projections
        :return: NumPy array containing projections
        """

        try: #try to open the projections as a stack
            projections = scipy.io.loadmat(filename)
            projections = np.array(projections[projections.keys()[0]])
        except: ##
             #check if the projections are in individual files
            flag = True
            filename_base, file_extension = os.path.splitext(filename)
            projectionCount = 1
            while flag: #first count the number of projections so the array can be initialized
                projectionCount = projectionCount
                nextFile = filename_base + str(projectionCount) + file_extension
                if os.path.isfile(nextFile):
                    projectionCount += 1
                else:
                    flag = False


            ## open first projection to get dimensions
            pj = scipy.io.loadmat(filename_base + str(1) + file_extension)
            pj = pj[projections.keys()[0]]
            dims = np.shape(pj)
            #initialize projection array
            projections = np.zeros((dims[0], dims[1], projectionCount),dtype=int)

            #now actually load in the tiff images
            for projNum in range(projectionCount):
                nextFile = filename_base + str(projNum) + file_extension
                pj = scipy.io.loadmat(filename_base + str(projNum) + file_extension)
                pj = pj[pj.keys()[0]]
                projections[:, :, projNum] = np.array(pj)

        return projections


    def readTIFF(filename):
        """
        * readTIFF *

        Read (possibly multiple) TIFF images into a NumPy array

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.

        :param filename: Name of TIFF file or TIFF file basename to read. If the filename is a base then
        #       the images must begin with the string contained in filename followed by consecutive integers with
        #       no zero padding, i.e. foo1.tiff, foo2.tiff,..., foo275.tiff
        :return: NumPy array containing projections
        """
        import functools
        from PIL import Image
        import os
        try:
            projections = np.array(Image.open(filename))
        except:
            flag = True
            filename_base, file_extension = os.path.splitext(filename)
            projectionCount = 1
            while flag: #first count the number of projections so the array can be initialized
                projectionCount = projectionCount
                nextFile = filename_base + str(projectionCount) + file_extension
                if os.path.isfile(nextFile):
                    projectionCount += 1
                else:
                    flag = False

            ## open first projection to get dimensions
            dims = np.shape(Image.open(filename_base + str(1) + file_extension))

            #initialize projection array
            projections = np.zeros((dims[0], dims[1], projectionCount),dtype=int)

            pool = Pool(4)
            func = functools.partial(readInTiffProjection, filename_base)
            pj = pool.map(func, range(projectionCount))
            for j  in range(projectionCount):
                projections[:, :, j] = pj[j]
            return projections

    def readInTiffProjection(filename_base, fileNumber):
        """
        * readInTiffProjection *

        Reads and returns a single TIFF image as a NumPy array

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.

        :param filename_base: Base filename of TIFF
        :param fileNumber: Image number
        :return: Image in a 2D NumPy array
        """
        from PIL import Image
        nextFile = filename_base + str(fileNumber) + '.tif'
        return np.array(Image.open(nextFile))

    def readMRC(filename, dtype=float,order="C"):
        """
        * readMRC *

        Read in a volume in .mrc file format. See http://bio3d.colorado.edu/imod/doc/mrc_format.txt

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.

        :param filename: Filename of .mrc
        :return: NumPy array containing the .mrc data
        """
        import struct
        headerIntNumber = 56
        sizeof_int = 4
        headerCharNumber = 800
        sizeof_char = 1
        with open(filename,'rb') as fid:
            int_header = struct.unpack('=' + 'i'*headerIntNumber, fid.read(headerIntNumber * sizeof_int))
            char_header = struct.unpack('=' + 'c'*headerCharNumber, fid.read(headerCharNumber * sizeof_char))
            dimx, dimy, dimz, data_flag= int_header[:4]
            if (data_flag == 0):
                datatype='u1'
            elif (data_flag ==1):
                datatype='i1'
            elif (data_flag ==2):
                datatype='f4'
            elif (data_flag ==3):
                datatype='c'
            elif (data_flag ==4):
                datatype='f4'
            elif (data_flag ==6):
                datatype='u2'
            else:
                raise ValueError("No supported datatype found!\n")
            return np.fromfile(file=fid, dtype=datatype,count=dimx*dimy*dimz).reshape((dimx,dimy,dimz),order=order).astype(dtype)

    def writeMRC(filename, arr, datatype='f4'):
        """
        * writeMRC *

        Write a volume to .mrc file format. See http://bio3d.colorado.edu/imod/doc/mrc_format.txt

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved

        :param filename: Filename of .mrc file to write
        :param arr: NumPy volume of data to write
        :param dtype: Type of data to write
        """
        dimx, dimy, dimz = np.shape(arr)
        if datatype != arr.dtype:
            arr = arr.astype(datatype)
        int_header = np.zeros(56,dtype='int32')

        if (datatype == 'u1'):
            data_flag = 0
        elif (datatype =='i1'):
            data_flag = 1
        elif (datatype =='f4'):
            data_flag = 2
        elif (datatype =='c'):
            data_flag = 3
        elif (datatype =='f4'):
            data_flag = 4
        elif (datatype =='u2'):
            data_flag = 6
        else:
            raise ValueError("No supported datatype found!\n")

        int_header[:4] = (dimx,dimy,dimz,data_flag)
        char_header = str(' '*800)
        with open(filename,'wb') as fid:
            fid.write(int_header.tobytes())
            fid.write(char_header)
            fid.write(arr.tobytes())



    class DisplayFigure:
        """
        * DisplayFigure *

        Helper class for displaying figures during reconstruction process

        Author: Alan (AJ) Pryor, Jr.
        Jianwei (John) Miao Coherent Imaging Group
        University of California, Los Angeles
        Copyright 2015-2016. All rights reserved.
        """
        def __init__(self):
            self.DisplayFigureON = False
            self.DisplayErrorFigureON = False
            self.displayFrequency = 5
            self.reconstructionDisplayWindowSize = 0


def toString(string):
    try:
        import genfire.gui.utility
        return genfire.gui.utility.toString(string)
    except ImportError:
        return str(string)

class ReconstructionParameters():

    """
    Helper class for containing reconstruction parameters
    """
    _supportedFiletypes = ['.tif', '.mrc', '.mat', '.npy']
    _supportedAngleFiletypes = ['.txt', '.mat', '.npy']
    def __init__(self):
        self.projectionFilename                  = ""
        self.angleFilename                       = ""
        self.supportFilename                     = ""
        self.resolutionExtensionSuppressionState = 1 #1 for resolution extension/suppression, 2 for off, 3 for just extension
        self.numIterations                       = 100
        self.displayFigure                       = DisplayFigure()
        self.oversamplingRatio                   = 3
        self.interpolationCutoffDistance         = 0.5
        self.isInitialObjectDefined              = False
        self.resultsFilename                     = os.path.join(os.getcwd(), 'results.mrc')
        self.useDefaultSupport                   = True
        self.calculateRfree                      = True
        self.initialObjectFilename               = None
        self.constraint_positivity               = True
        self.constraint_support                  = True
        self.griddingMethod                      = "FFT"
        self.enforceResolutionCircle             = True
        self.permitMultipleGridding              = True

    def checkParameters(self): #verify file extensions are supported
        import os
        parametersAreGood = 1

        projection_extension = os.path.splitext(self.projectionFilename)
        if projection_extension[1] not in ReconstructionParameters._supportedFiletypes \
                or not os.path.isfile(self.projectionFilename):
            parametersAreGood = 0

        angle_extension = os.path.splitext(self.angleFilename)
        if angle_extension[1] not in ReconstructionParameters._supportedAngleFiletypes \
                or not os.path.isfile(self.angleFilename):
            parametersAreGood = 0

        if not self.useDefaultSupport:
            if self.supportFilename != "": #empty support filename is okay, as this will trigger generation of a default support
                support_extension = os.path.splitext(self.supportFilename)
                if support_extension[1] not in ReconstructionParameters._supportedFiletypes \
                        or not os.path.isfile(self.supportFilename):
                    parametersAreGood = 0

        if not self.getResultsFilename():
            parametersAreGood = 0
        return parametersAreGood

    # Define setters/getters. It's not very pythonic to do so, but  I came from C++ and wrote
    # this before I knew better. In any case it doesn't affect much.

    def setProjectionFilename(self, projectionFilename):
        if projectionFilename:
            self.projectionFilename = os.path.join(os.getcwd(), toString(projectionFilename))

    def getProjectionFilename(self):
        return self.projectionFilename

    def setAngleFilename(self, angleFilename):
        if angleFilename:
            self.angleFilename = os.path.join(os.getcwd(), toString(angleFilename))

    def getAngleFilename(self):
        return self.angleFilename

    def setSupportFilename(self, supportFilename):
        if supportFilename:
            self.supportFilename = os.path.join(os.getcwd(), toString(supportFilename))

    def getSupportFilename(self):
        return self.supportFilename

    def setResultsFilename(self, resultsFilename):
        if resultsFilename:
            self.resultsFilename = os.path.join(os.getcwd(), toString(resultsFilename))

    def getResultsFilename(self):
        return self.resultsFilename

    def setInitialObjectFilename(self, initialObjectFilename):
        self.initialObjectFilename = os.path.join(os.getcwd(), toString(initialObjectFilename))
        self.isInitialObjectDefined = True

    def getInitialObjectFilename(self):
        if self.CheckIfInitialObjectIsDefined():
            return self.initialObjectFilename
        else:
            pass

    def CheckIfInitialObjectIsDefined(self):
        return self.isInitialObjectDefined

    def setResolutionExtensionSuppressionState(self, state):
        self.resolutionExtensionSuppressionState = state

    def getResolutionExtensionSuppressionState(self):
        return self.resolutionExtensionSuppressionState

    def setNumberOfIterations(self,numIterations):
        self.numIterations = numIterations

    def getNumberOfIterations(self):
        return self.numIterations

    def toggleDisplayFigure(self): # whether or not to display figure during reconstruction
        if self.displayFigure.DisplayFigureON:
            self.displayFigure.DisplayFigureON = False
        else:
            self.displayFigure.DisplayFigureON = True

        if self.displayFigure.DisplayErrorFigureON:
            self.displayFigure.DisplayErrorFigureON = False
        else:
            self.displayFigure.DisplayErrorFigureON = True

    def getDisplayFigure(self):
        return self.displayFigure

    def setOversamplingRatio(self, oversamplingRatio):
        self.oversamplingRatio = int(oversamplingRatio)

    def getOversamplingRatio(self):
        return self.oversamplingRatio

    def setInterpolationCutoffDistance(self, interpolationCutoffDistance):
        self.interpolationCutoffDistance = float(interpolationCutoffDistance)

    def getInterpolationCutoffDistance(self):
        return self.interpolationCutoffDistance

class GenfireReconstructor():
    """
    Primary class for running GENFIRE Reconstructions
    """
    def __init__(self, projectionFilename="", angleFilename="", supportFilename="", initialObjectFilename=None,
                 resultsFilename=os.path.join(os.getcwd(), 'results.mrc'), resolutionExtensionSuppressionState=1,
                 numIterations=100, oversamplingRatio=3, interpolationCutoffDistance=0.5, useDefaultSupport=True,
                 calculateRfree=True, constraint_positivity=True, constraint_support=True, griddingMethod="FFT",
                 enforceResolutionCircle=True, permitMultipleGridding=True, verbose=True):

        """
        :param projectionFilename: (string) file containing projections
        :param angleFilename: (string) file containing Euler angles
        :param supportFilename: (string) file containing support
        :param resultsFilename: (string) file in which to store reconstruction volume
        :param initialObjectFilename: optional filename containing intial object
        :param resolutionExtensionSuppressionState: (int) 1: use resolution extension/suppression;
        2: do not use; 3: extension only
        :param numIterations: (int) number of GENFIRE iterations
        :param oversamplingRatio: (int) oversampling ratio; determines how much zero-padding is applied prior to
         gridding
        :param interpolationCutoffDistance: (float) distance in pixels within which to consider data for gridding
        :param useDefaultSupport: (bool) create a default support; defined as a cube with dimensions identical to
         input projections
        :param calculateRfree: (bool) whether or not to compute Rfree
        :param constraint_positivity: (bool) whether to enforce positivity constraint
        :param constraint_support: (bool) whether to enforce support constraint
        :param griddingMethod: (string) "FFT" or "DFT"
        :param enforceResolutionCircle: (bool) whether to limit gridded data to within Nyquist frequency
        :param permitMultipleGridding: (bool) whether to allow a given measured datapoint to be included in calculation
         of multiple Fourier grid points
        :param verbose: (bool) prints additional information about the reconstruction
        """

        self.reconstruction_ = None
        self.errK_ = None
        self.R_free_bybin_ = None
        self.R_free_total_ = None

        self.params = ReconstructionParameters()
        self.params.projectionFilename = projectionFilename
        self.params.angleFilename = angleFilename
        self.params.supportFilename = supportFilename
        self.params.resultsFilename = resultsFilename
        self.params.resolutionExtensionSuppressionState = resolutionExtensionSuppressionState #1 for resolution extension/suppression, 2 for off, 3 for just extension
        self.params.numIterations = numIterations
        self.params.oversamplingRatio = oversamplingRatio
        self.params.interpolationCutoffDistance = interpolationCutoffDistance
        self.params.isInitialObjectDefined = False
        self.params.useDefaultSupport = useDefaultSupport
        self.params.calculateRfree = calculateRfree
        self.params.initialObjectFilename = initialObjectFilename
        self.params.constraint_positivity = constraint_positivity
        self.params.constraint_support = constraint_support
        self.params.griddingMethod = griddingMethod
        self.params.enforceResolutionCircle = enforceResolutionCircle
        self.params.permitMultipleGridding = permitMultipleGridding
        self.verbose = verbose

    def printGenfire(self):
        print("""
       ______  ________  ____  _____  ________  _____  _______     ________
     .' ___  ||_   __  ||_   \|_   _||_   __  ||_   _||_   __ \   |_   __  |
    / .'   \_|  | |_ \_|  |   \ | |    | |_ \_|  | |    | |__) |    | |_ \_|
    | |   ____  |  _| _   | |\ \| |    |  _|     | |    |  __ /     |  _| _
    \ `.___]  |_| |__/ | _| |_\   |_  _| |_     _| |_  _| |  \ \_  _| |__/ |
     `._____.'|________||_____|\____||_____|   |_____||____| |___||________|

        """)

    def printParams(self):
        print("projectionFilename = {}".format(self.params.projectionFilename))
        print("angleFilename = {}".format(self.params.angleFilename))
        print("supportFilename = {}".format(self.params.supportFilename))
        print("resultsFilename = {}".format(self.params.resultsFilename))
        print("resolutionExtensionSuppressionState = {}".format(self.params.resolutionExtensionSuppressionState))
        print("numIterations = {}".format(self.params.numIterations))
        print("oversamplingRatio = {}".format(self.params.oversamplingRatio))
        print("interpolationCutoffDistance = {}".format(self.params.interpolationCutoffDistance))
        print("useDefaultSupport = {}".format(self.params.useDefaultSupport))
        print("calculateRfree = {}".format(self.params.calculateRfree))
        print("initialObjectFilename = {}".format(self.params.initialObjectFilename))
        print("constraint_positivity = {}".format(self.params.constraint_positivity))
        print("constraint_support = {}".format(self.params.constraint_support))
        print("enforceResolutionCircle = {}".format(self.params.enforceResolutionCircle))
        print("permitMultipleGridding = {}".format(self.params.permitMultipleGridding))

    def reconstruct(self):
        if self.verbose:
            self.printGenfire()
            self.printParams()
        projections = genfire.fileio.loadProjections(self.params.projectionFilename) # load projections into a 3D numpy array

        # get dimensions of array and determine the array size after padding
        dims = np.shape(projections)
        paddedDim = dims[0] * self.params.oversamplingRatio
        padding = int((paddedDim-dims[0])/2)

        # load the support, or generate one if none was provided
        if self.params.useDefaultSupport or self.params.supportFilename == "":
            support = np.ones((dims[0],dims[0],dims[0]),dtype=float)
        else:
            support = (genfire.fileio.readVolume(self.params.supportFilename) != 0).astype(bool)

        # now zero-pad to match the oversampling ratio
        support = np.pad(support,((padding,padding),(padding,padding),(padding,padding)),'constant')
        projections = np.pad(projections,((padding,padding),(padding,padding),(0,0)),'constant')

        #load initial object, or initialize it to zeros if none was given
        if self.params.initialObjectFilename is not None and os.path.isfile(self.params.initialObjectFilename):
            initialObject = genfire.fileio.readVolume(self.params.initialObjectFilename)
            initialObject = np.pad(initialObject,((padding,padding),(padding,padding),(padding,padding)),'constant')
        else:
            initialObject = np.zeros_like(support)

        euler_angles = genfire.fileio.loadAngles(self.params.angleFilename)
        if np.shape(euler_angles)[1] > 3:
            raise ValueError("Error! Dimension of angles incorrect.")
        if np.shape(euler_angles)[1] == 1:
            tmp = np.zeros([np.shape(euler_angles)[1], 3])
            tmp[1, :] = euler_angles
            angles = tmp
            del tmp

        # grid the projections
        if self.params.griddingMethod == "DFT":
            measuredK = genfire.reconstruct.fillInFourierGrid_DFT(projections,
                                                                  euler_angles,
                                                                  self.params.interpolationCutoffDistance,
                                                                  self.params.enforceResolutionCircle)
        else:
            measuredK = genfire.reconstruct.fillInFourierGrid(projections,
                                                              euler_angles,
                                                              self.params.interpolationCutoffDistance,
                                                              self.params.enforceResolutionCircle,
                                                              self.params.permitMultipleGridding)

        measuredK = np.fft.ifftshift(measuredK)

        # create a map of the spatial frequency to be used to control resolution extension/suppression behavior
        K_indices = genfire.utility.generateKspaceIndices(support)
        K_indices = np.fft.fftshift(K_indices)
        resolutionIndicators = np.zeros_like(K_indices)
        resolutionIndicators[measuredK != 0] = 1-K_indices[measuredK != 0]

        # if calculating Rfree, setup some infrastructure
        if self.params.calculateRfree:
            R_freeInd_complexX = []
            R_freeInd_complexY = []
            R_freeInd_complexZ = []
            R_freeVals_complex = []
            shell_thickness_pixels = 1 # pixel thickness of an individual shell of Rfree points
            numberOfBins = int(round(dims[0]/2/shell_thickness_pixels)) # number of frequency bins. Rfree will be tracked within each shell separately
            percentValuesForRfree = 0.05 # percentage of measured points to withhold
            spatialFrequencyForRfree = np.linspace(0,1,numberOfBins+1)
            K_indicesSmall =(K_indices)[:, :, 0:(np.shape(measuredK)[-1]//2+1)]

            for shellNum in range(0,numberOfBins):
                # collect relevant points
                measuredPointInd_complex = np.where((measuredK[:, :, 0:(np.shape(measuredK)[-1]//2+1)] != 0) & (K_indicesSmall>=(spatialFrequencyForRfree[shellNum])) & (K_indicesSmall<=(spatialFrequencyForRfree[shellNum+1])))

                # randomly shuffle
                shuffledPoints = np.random.permutation(np.shape(measuredPointInd_complex)[1])
                measuredPointInd_complex = (measuredPointInd_complex[0][shuffledPoints], measuredPointInd_complex[1][shuffledPoints], measuredPointInd_complex[2][shuffledPoints])

                # determine how many values to take
                cutoffInd_complex = np.floor(np.shape(measuredPointInd_complex)[1] * percentValuesForRfree).astype(int)

                # collect the Rfree values and coordinates
                R_freeInd_complexX.append(measuredPointInd_complex[0][:cutoffInd_complex])
                R_freeInd_complexY.append(measuredPointInd_complex[1][:cutoffInd_complex])
                R_freeInd_complexZ.append(measuredPointInd_complex[2][:cutoffInd_complex])
                R_freeVals_complex.append(measuredK[R_freeInd_complexX[shellNum], R_freeInd_complexY[shellNum], R_freeInd_complexZ[shellNum] ])

                # delete the points from the measured data
                measuredK[R_freeInd_complexX[shellNum], R_freeInd_complexY[shellNum], R_freeInd_complexZ[shellNum]] = 0

            # create tuple of coordinates
            R_freeInd_complex = [[R_freeInd_complexX], [R_freeInd_complexY], [R_freeInd_complexZ]]
            del R_freeInd_complexX
            del R_freeInd_complexY
            del R_freeInd_complexZ
        else:
            R_freeInd_complex = []
            R_freeVals_complex = []

        if self.params.resolutionExtensionSuppressionState==1: # resolution extension/suppression
            constraintEnforcementDelayIndicators = np.array(np.concatenate((np.arange(0.95, -.25, -0.15), np.arange(-0.15, .95, .1)), axis=0))
        elif self.params.resolutionExtensionSuppressionState==2:# no resolution extension or suppression
            constraintEnforcementDelayIndicators = np.array([-999, -999, -999, -999])
        elif self.params.resolutionExtensionSuppressionState==3:# resolution extension only
            constraintEnforcementDelayIndicators = np.concatenate((np.arange(0.95, -.15, -0.15),[-0.15, -0.15, -0.15]))
        else:
            print("Warning! Input resolutionExtensionSuppressionState does not match an available option. Deactivating dynamic constraint enforcement and continuing.\n")
            constraintEnforcementDelayIndicators = np.array([-999, -999, -999, -999])


        results = reconstruct(self.params.numIterations,
                              np.fft.fftshift(initialObject),
                              np.fft.fftshift(support),
                              (measuredK)[:, :, 0:(np.shape(measuredK)[-1] // 2 + 1)],
                              (resolutionIndicators)[:, :, 0:(np.shape(measuredK)[-1] // 2 + 1)],
                              constraintEnforcementDelayIndicators,
                              R_freeInd_complex,
                              R_freeVals_complex,
                              self.params.displayFigure,
                              self.params.constraint_positivity,
                              self.params.constraint_support)
        ncBig = paddedDim//2
        n2 = dims[0]//2
        results['reconstruction'] = results['reconstruction'][ncBig-n2:ncBig+n2,ncBig-n2:ncBig+n2,ncBig-n2:ncBig+n2]
        return results
        # self.reconstruction_ = results['reconstruction']
        # self.errK_ = results['R_free_bybin']
        # if "R_freeInd_complex" in results.keys():
        #     self.R_free_bybin_ = results['R_free_bybin']
        #     self.R_free_total_ = results['R_free_total']


"""
* genfire.gui.ProjectionCalculator *

The projection calculator module


Author: Alan (AJ) Pryor, Jr.
Jianwei (John) Miao Coherent Imaging Group
University of California, Los Angeles
Copyright 2015-2016. All rights reserved.
"""


from __future__ import print_function

from PyQt5 import QtCore, QtGui, QtWidgets
import os
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from genfire.gui import ProjectionCalculator_MainWindow
from genfire.gui import CalculateProjectionSeries_Dialog
from genfire.gui.utility import toString, toQString, toInt, toFloat
from genfire.utility import *
import genfire

def validatedStringToFloat(string):
    try:
        val = toFloat(string)
    except ValueError:
        return 0.0
    return val

def validateFloatFromLineEdit(self, line_edit):
    try:
        val = str(line_edit.text())
    except ValueError:
        line_edit.setText("1.0")

class ProjectionCalculator(QtWidgets.QMainWindow):
    model_loading_signal = QtCore.pyqtSignal()
    update_filenames_signal = QtCore.pyqtSignal()
    emit_message_signal = QtCore.pyqtSignal(str)
    def __init__(self, parent=None):
        super(ProjectionCalculator, self).__init__()
        self.ui = ProjectionCalculator_MainWindow.Ui_ProjectionCalculator()
        self.ui.setupUi(self) # buid the interface
        self.parent = parent
        self.calculationParameters = ProjectionCalculationParameters()

        self.ui.lineEdit_modelFile.setText(toQString(os.getcwd()))
        self.ui.lineEdit_modelFile.editingFinished.connect(self.setModelFilename_fromLineEdit)

        ## Push Buttons
        self.ui.btn_selectModel.clicked.connect(self.selectModelFile)

        ## Sliders
        self.ui.verticalSlider_phi.setValue(0)
        self.ui.verticalSlider_phi.setMinimum(0)
        self.ui.verticalSlider_phi.setMaximum(3600)
        self.ui.verticalSlider_phi.valueChanged.connect(self.setPhiLineEditValue)

        self.ui.verticalSlider_theta.setValue(0)
        self.ui.verticalSlider_theta.setMinimum(0)
        self.ui.verticalSlider_theta.setMaximum(3600)
        self.ui.verticalSlider_theta.valueChanged.connect(self.setThetaLineEditValue)

        self.ui.verticalSlider_psi.setValue(0)
        self.ui.verticalSlider_psi.setMinimum(0)
        self.ui.verticalSlider_psi.setMaximum(3600)
        self.ui.verticalSlider_psi.valueChanged.connect(self.setPsiLineEditValue)

        ## Line edits
        self.ui.lineEdit_phi.setText(toQString('0'))
        self.ui.lineEdit_theta.setText(toQString('0'))
        self.ui.lineEdit_psi.setText(toQString('0'))
        self.ui.lineEdit_phi.editingFinished.connect(self.setPhiSliderValue)
        self.ui.lineEdit_theta.editingFinished.connect(self.setThetaSliderValue)
        self.ui.lineEdit_psi.editingFinished.connect(self.setPsiSliderValue)

        self.calculateProjections_Dialog = CalculateProjectionSeries_popup(self.calculationParameters)
        self.ui.btn_go.clicked.connect(self.showDialog)
        self.calculateProjections_Dialog.ui.buttonBox.accepted.connect(self.readyToCalculateProjections)
        self.ui.btn_go.setEnabled(False)

        # Create figures for embedded plotting
        self.figure = plt.figure(3)
        self.figure.clf() # clear figure in case it was rendered somewhere else previously
        self.canvas = FigureCanvas(self.figure)
        self.navigationToolbar = NavigationToolbar(self.canvas, self)
        self.ui.verticalLayout_figure.addWidget(self.navigationToolbar)
        self.ui.verticalLayout_figure.addWidget(self.canvas)
        self.ax = self.figure.add_subplot(111)
        self.ax.axes.get_xaxis().set_visible(False)
        self.ax.axes.get_yaxis().set_visible(False)

        self.ui.btn_clearModel.clicked.connect(self.clearModel)

    def showDialog(self):
        self.calculateProjections_Dialog.show()

    def readyToCalculateProjections(self):
        self.emit_message_signal.emit("GENFIRE: Calculating projections...")
        self.calculation_thread = ProjectionCalculator_thread(self)
        self.calculation_thread.finished.connect(self.readyToClose)
        self.calculation_thread.start()
        self.parent.raise_()

    def readyToClose(self):
        self.update_filenames_signal.emit()
        self.close()

    def calculateProjections(self):
        if self.calculateProjections_Dialog.status:
            self.calculationParameters.modelFilename = toString(self.calculationParameters.modelFilename)
            if not self.calculationParameters.modelLoadedFlag:

                self.calculationParameters.model = genfire.fileio.readVolume(self.calculationParameters.modelFilename)
                self.calculationParameters.oversamplingRatio = self.calculationParameters.oversamplingRatio
                self.calculationParameters.dims = np.shape(self.calculationParameters.model)
                paddedDim = self.calculationParameters.dims[0] * self.calculationParameters.oversamplingRatio
                padding = int((paddedDim-self.calculationParameters.dims[0])/2)
                self.calculationParameters.model = np.pad(self.calculationParameters.model,((padding,padding),(padding,padding),(padding,padding)),'constant')
                self.calculationParameters.model = fftn_fftshift(self.calculationParameters.model)
                self.calculationParameters.interpolator = genfire.utility.getProjectionInterpolator(self.calculationParameters.model)
            self.calculationParameters.ncOut = np.shape(self.calculationParameters.model)[0]//2


            if self.calculationParameters.angleFileProvided:
                self.calculationParameters.angleFilename = toString(self.calculationParameters.angleFilename)
                self.calculationParameters.outputFilename = toString(self.calculationParameters.outputFilename)
                angles = np.loadtxt(toString(self.calculationParameters.angleFilename))
                phi = angles[:, 0]
                theta = angles[:, 1]
                psi = angles[:, 2]
                filename = self.calculationParameters.outputFilename +'.npy'
                projections = np.zeros((self.calculationParameters.dims[0],self.calculationParameters.dims[1],np.size(phi)),dtype=float)
                if self.calculationParameters.interpolator is None:
                    self.calculationParameters.interpolator = genfire.utility.getProjectionInterpolator(self.calculationParameters.model)
                for projNum in range(0,np.size(phi)):
                    pj = genfire.utility.calculateProjection_interp_fromInterpolator(self.calculationParameters.interpolator, phi[projNum], theta[projNum], psi[projNum], np.shape(self.calculationParameters.model))
                    projections[:, :, projNum] = pj[self.calculationParameters.ncOut-int(self.calculationParameters.dims[0]/2):self.calculationParameters.ncOut+int(self.calculationParameters.dims[0]/2), self.calculationParameters.ncOut-int(self.calculationParameters.dims[1]/2):self.calculationParameters.ncOut+int(self.calculationParameters.dims[1]/2)]

                genfire.fileio.writeVolume(filename, projections)
            else:
                if self.calculationParameters.interpolator is None:
                    self.calculationParameters.interpolator = genfire.utility.getProjectionInterpolator(self.calculationParameters.model)
                phi = self.calculationParameters.phi
                psi = self.calculationParameters.psi
                theta = np.arange(self.calculationParameters.thetaStart, \
                                  self.calculationParameters.thetaStop + 1e-10, \
                                  self.calculationParameters.thetaStep)
                projections = np.zeros((self.calculationParameters.dims[0],self.calculationParameters.dims[1],np.size(theta)),dtype=float)
                for i, current_theta in enumerate(theta):

                    pj = genfire.utility.calculateProjection_interp_fromInterpolator(self.calculationParameters.interpolator, phi, current_theta, psi, np.shape(self.calculationParameters.model))
                    projections[:, :, i] = pj[self.calculationParameters.ncOut-int(self.calculationParameters.dims[0]/2):self.calculationParameters.ncOut+int(self.calculationParameters.dims[0]/2), self.calculationParameters.ncOut-int(self.calculationParameters.dims[1]/2):self.calculationParameters.ncOut+int(self.calculationParameters.dims[1]/2)]
                filename = self.calculationParameters.outputFilename
                filename = toString(filename)
                # projections[ projections<0 ] = 0
                genfire.fileio.writeVolume(filename, projections)
                if self.calculationParameters.writeAnglesFlag:
                    output_filename_base, ext  = os.path.splitext(toString(self.calculationParameters.outputFilename))
                    output_angle_filename      = output_filename_base + "_euler_angles.txt"
                    if os.path.isfile(output_angle_filename):
                        print("WARNING! Filename {} already exists and will be overwritten.".format(output_angle_filename))
                    self.calculationParameters.outputAngleFilename = output_angle_filename
                    num_projections = np.size(theta)
                    phi = np.repeat(phi, num_projections)
                    psi = np.repeat(psi, num_projections)
                    with open(output_angle_filename,'w') as fid:
                        for euler in zip(phi, theta, psi):
                            string = str(euler[0]) + ' ' + str(euler[1]) + ' ' + str(euler[2]) + '\n'
                            fid.write(string)
                    print("Successfully calculated {}.".format(output_angle_filename))


            print("Successfully calculated {}.".format(filename))

    def clearModel(self):
        self.calculationParameters.model         = None
        self.calculationParameters.interpolator  = None
        self.calculationParameters.modelFilename = None
        self.clearFigure()
        self.ui.btn_go.setEnabled(False)

    def displayFigure(self):
        if self.calculationParameters.model is not None:
            pj = genfire.utility.calculateProjection_interp_fromInterpolator(self.calculationParameters.interpolator, self.calculationParameters.phi, self.calculationParameters.theta, self.calculationParameters.psi, np.shape(self.calculationParameters.model))
            pj = pj[self.calculationParameters.ncOut-int(int(self.calculationParameters.dims[0]/2)):self.calculationParameters.ncOut+int(int(self.calculationParameters.dims[0]/2)), self.calculationParameters.ncOut-int(int(self.calculationParameters.dims[1]/2)):self.calculationParameters.ncOut+int(int(self.calculationParameters.dims[1]/2))]
            pj[ pj<0 ] = 0
            self.showProjection(pj)
        else:
            self.clearFigure()

    def updateFigure(self):
        if self.calculationParameters.interpolator is None:
            self.calculationParameters.interpolator = genfire.utility.getProjectionInterpolator(self.calculationParameters.model)
        pj = genfire.utility.calculateProjection_interp_fromInterpolator(self.calculationParameters.interpolator, self.calculationParameters.phi, self.calculationParameters.theta, self.calculationParameters.psi, np.shape(self.calculationParameters.model))
        pj = pj[self.calculationParameters.ncOut-int(self.calculationParameters.dims[0]/2):self.calculationParameters.ncOut+int(self.calculationParameters.dims[0]/2), self.calculationParameters.ncOut-int(self.calculationParameters.dims[1]/2):self.calculationParameters.ncOut+int(self.calculationParameters.dims[1]/2)]
        pj[ pj<0 ] = 0
        self.ax.imshow(pj)
        self.canvas.draw()


    def showProjection(self, pj):
        self.ax.imshow(pj)
        self.canvas.draw()

    def clearFigure(self):
        self.figure.clf()
        self.canvas.draw()
        self.ax = self.figure.add_subplot(111)
        self.ax.axes.get_xaxis().set_visible(False)
        self.ax.axes.get_yaxis().set_visible(False)

    def setPhiSliderValue(self):
        value = validatedStringToFloat(self.ui.lineEdit_phi.text())
        value = int(value * 10)
        self.ui.verticalSlider_phi.setValue(value)

    def setThetaSliderValue(self):
        value = validatedStringToFloat(self.ui.lineEdit_theta.text())
        value = int(value * 10)
        self.ui.verticalSlider_theta.setValue(value)

    def setPsiSliderValue(self):
        value = validatedStringToFloat(self.ui.lineEdit_psi.text())
        value = int(value * 10)
        self.ui.verticalSlider_psi.setValue(value)


    def setPhiLineEditValue(self, value):
        self.ui.lineEdit_phi.setText(toQString(float(value)/10))
        self.calculationParameters.phi = float(value)/10
        self.updateFigure()

    def setThetaLineEditValue(self, value):
        self.ui.lineEdit_theta.setText(toQString(float(value)/10))
        self.calculationParameters.theta = float(value)/10
        self.updateFigure()

    def setPsiLineEditValue(self, value):
        self.ui.lineEdit_psi.setText(toQString(float(value)/10))
        self.calculationParameters.psi = float(value)/10
        self.updateFigure()

    def setNumberOfProjections(self, number):
        self.calculationParameters.numberOfProjections = toInt(number)

    def setPhiStart(self, phiStart):
        self.calculationParameters.phiStart = validatedStringToFloat(phiStart)

    def setThetaStart(self, thetaStart):
        self.calculationParameters.thetaStart = validatedStringToFloat(thetaStart)

    def setPsiStart(self, psiStart):
        self.calculationParameters.psiStart = validatedStringToFloat(psiStart)

    def setPhiStep(self, phiStep):
        self.calculationParameters.phiStep = validatedStringToFloat(phiStep)

    def setThetaStep(self, thetaStep):
        self.calculationParameters.thetaStep = validatedStringToFloat(thetaStep)

    def setPsiStep(self, psiStep):
        self.calculationParameters.psiStep = validatedStringToFloat(psiStep)

    def selectOutputDirectory(self):
        dirname, _ = QtWidgets.QFileDialog.getExistingDirectory(QtWidgets.QFileDialog(), "Select File Containing Angles",options=QtWidgets.QFileDialog.ShowDirsOnly)
        if dirname:
            self.calculationParameters.outputFilename =  dirname
            # self.ui.lineEdit_outputFilename.setText(toQString(dirname))

    def setModelFilename_fromLineEdit(self):
        filename = self.ui.lineEdit_modelFile.text()
        if os.path.isfile(toString(filename)) and filename != self.calculationParameters.modelFilename:
            self.setModelFilename(toString(filename))

    def setModelFilename(self, filename):
        self.calculationParameters.modelFilename = filename
        self.ui.lineEdit_modelFile.setText(toQString(filename))
        self.loadModel(filename)
        self.displayFigure()

    def selectModelFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select File Containing Model",filter="Volume files (*.mrc *.mat *.npy);;MATLAB files (*.mat);;TIFF images (*.tif *.tiff);;MRC (*.mrc);;numpy (*.npy);;All Files (*)")
        print(filename)
        if os.path.isfile(toString(filename)):
            self.ui.btn_go.setEnabled(True)
            self.setModelFilename(filename)

    def loadModel(self, filename):
        self.calculationParameters.oversamplingRatio, ok = QtWidgets.QInputDialog.getInt(self, "Select Oversampling Ratio","Oversampling Ratio = ",value=3,min=1,max=10,step=1)
        from threading import Thread
        t = Thread(target=lambda:print("Loading Model..."))
        t.start()
        self.ui.btn_go.setEnabled(True)
        self.calculationParameters.model = genfire.fileio.readVolume(toString(filename))
        self.calculationParameters.dims = np.shape(self.calculationParameters.model)
        self.calculationParameters.paddedDim = self.calculationParameters.dims[0] * self.calculationParameters.oversamplingRatio
        padding = int((self.calculationParameters.paddedDim-self.calculationParameters.dims[0])/2)
        self.calculationParameters.model = np.pad(self.calculationParameters.model,((padding,padding),(padding,padding),(padding,padding)),'constant')
        self.calculationParameters.model = fftn_fftshift(self.calculationParameters.model)
        self.calculationParameters.ncOut = int(np.shape(self.calculationParameters.model)[0]//2)
        self.calculationParameters.interpolator = genfire.utility.getProjectionInterpolator(self.calculationParameters.model)
        t.join()

    def setAngleFilename(self, filename):
        if filename:
            self.calculationParameters.angleFilename = filename

    def toggleSaveProjections(self):
        print  (GENFIRE_ProjectionCalculator.calculationParameters.modelFilename)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.angleFilename)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.outputFilename)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.outputFilesFlag)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.phiStart)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.thetaStart)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.psiStart)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.phiStep)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.thetaStep)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.psiStep)
        print  (GENFIRE_ProjectionCalculator.calculationParameters.numberOfProjections)


        if self.calculationParameters.outputFilesFlag:
            self.calculationParameters.outputFilesFlag = False
        else:
            self.calculationParameters.outputFilesFlag = True

    def setOutputFilename(self, filename):
        if filename:
            self.calculationParameters.outputFilename = filename


class ProjectionCalculationParameters:
    oversamplingRatio = 3
    def __init__(self):
        self.modelFilename              = toQString('')
        self.angleFilename              = toQString('')
        self.outputFilename             = toQString('')
        self.outputAngleFilename        = toQString('')
        self.outputFilesFlag            = False
        self.angleFileProvided          = False
        self.modelFilenameProvided      = False
        self.modelLoadedFlag            = False
        self.writeAnglesFlag            = True
        self.interpolator               = None
        self.model                      = None
        self.ncOut                      = None
        self.phi                        = 0.0
        self.theta                      = 0.0
        self.thetaStart                 = 0.0
        self.thetaStep                  = 3.0
        self.thetaStop                  = 180.0
        self.psi                        = 0.0
        self.calculationMethod          = "FFT"

class CalculateProjectionSeries_popup(QtWidgets.QDialog):
    def __init__(self, calculation_parameters = ProjectionCalculationParameters()):
        super(CalculateProjectionSeries_popup, self).__init__()
        self.calculationParameters = calculation_parameters

        self.ui = CalculateProjectionSeries_Dialog.Ui_CalculateProjectionSeries_Dialog()
        self.ui.setupUi(self)
        self.setModal(True)

        import os
        from functools import partial
        self.calculationParameters.outputFilename = os.path.abspath(os.getcwd() + '/projections.mrc')
        self.ui.lineEdit_outputFilename.setText(self.calculationParameters.outputFilename)
        self.ui.lineEdit_phi.setText(toQString(str(self.calculationParameters.phi)))
        self.ui.lineEdit_psi.setText(toQString(str(self.calculationParameters.psi)))
        self.ui.lineEdit_thetaStart.setText(toQString(str(self.calculationParameters.thetaStart)))
        self.ui.lineEdit_thetaStep.setText(toQString(str(self.calculationParameters.thetaStep)))
        self.ui.lineEdit_thetaStop.setText(toQString(str(self.calculationParameters.thetaStop)))
        self.ui.lineEdit_angleFile.textEdited.connect(self.setAngleFilename_fromLineEdit)
        self.ui.btn_selectAngleFile.clicked.connect(self.selectAngleFile)
        self.ui.lineEdit_phi.textEdited.connect(self.setPhi)
        self.ui.lineEdit_psi.textEdited.connect(self.setPsi)
        self.ui.lineEdit_thetaStart.textEdited.connect(self.setThetaStart)
        self.ui.lineEdit_thetaStep.textEdited.connect(self.setThetaStep)
        self.ui.lineEdit_thetaStop.textEdited.connect(self.setThetaStop)

        self.ui.lineEdit_outputFilename.textEdited.connect(self.setOutputFilename)
        self.ui.checkBox_saveAngles.setChecked(True)
        self.ui.checkBox_saveAngles.toggled.connect(self.toggleSaveAngles)
        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.Ok).setText("Calculate Projections")
        self.ui.buttonBox.accepted.connect(partial(self.setStatus, 1))

        # validations
        self.ui.lineEdit_phi.editingFinished.connect(
            partial(self.validateFloatFromLineEdit, self.ui.lineEdit_phi))
        self.ui.lineEdit_psi.editingFinished.connect(
            partial(self.validateFloatFromLineEdit, self.ui.lineEdit_psi))
        self.ui.lineEdit_thetaStart.editingFinished.connect(
            partial(self.validateFloatFromLineEdit, self.ui.lineEdit_thetaStart))
        self.ui.lineEdit_thetaStep.editingFinished.connect(
            partial(self.validateFloatFromLineEdit, self.ui.lineEdit_thetaStep))
        self.ui.lineEdit_thetaStop.editingFinished.connect(
            partial(self.validateFloatFromLineEdit, self.ui.lineEdit_thetaStop))

        self.status = 0

    def validateFloatFromLineEdit(self, line_edit):
        try:
            val = toFloat(line_edit.text())
        except ValueError:
            line_edit.setText("1.0")

    def setStatus(self,status):
        self.status = status

    def setAngleFilename_fromLineEdit(self):
        filename = self.ui.lineEdit_angleFile.text()
        if os.path.isfile(toString(filename)):
            self.calculationParameters.angleFilename = filename
            self.calculationParameters.angleFileProvided = True
            self.ui.lineEdit_angleFile.setText(toQString(filename))
            self.disableAngleWidgets()

    # def setCalculationMethod(self):
    #     if self.ui.radioButton_FFT.isChecked():
    #         self.ui.radioButton_DFT.setChecked(False)
    #         self.calculationParameters.calculationMethod = "FFT"
    #     else:
    #         self.ui.radioButton_FFT.setChecked(False)
    #         self.calculationParameters.calculationMethod = "DFT"

    def selectAngleFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(QtWidgets.QFileDialog(), "Select File Containing Angles",filter="txt files (*.txt);;All Files (*)")
        if filename:
            self.calculationParameters.angleFilename = filename
            self.calculationParameters.angleFileProvided = True
            self.ui.lineEdit_angleFile.setText(toQString(filename))
            self.disableAngleWidgets()
    def toggleSaveAngles(self):
        if self.ui.checkBox_saveAngles.isChecked():
            self.calculationParameters.writeAnglesFlag = True
        else:
            self.calculationParameters.writeAnglesFlag = False
    def disableAngleWidgets(self):
        self.ui.lineEdit_thetaStart.setEnabled(True)
        self.ui.lineEdit_thetaStep.setDisabled(True)
        self.ui.lineEdit_thetaStop.setDisabled(True)
        self.ui.lineEdit_phi.setDisabled(True)
        self.ui.lineEdit_psi.setDisabled(True)

        self.ui.lineEdit_thetaStart.setStyleSheet("background-color: gray")
        self.ui.lineEdit_thetaStep.setStyleSheet("background-color: gray")
        self.ui.lineEdit_thetaStop.setStyleSheet("background-color: gray")
        self.ui.lineEdit_phi.setStyleSheet("background-color: gray")
        self.ui.lineEdit_psi.setStyleSheet("background-color: gray")
        self.ui.checkBox_saveAngles.setChecked(False)
        self.ui.checkBox_saveAngles.setEnabled(False)


    def setPhi(self, angle):
        self.calculationParameters.phi = validatedStringToFloat(angle)

    def setPsi(self, angle):
        self.calculationParameters.psi = validatedStringToFloat(angle)

    def setThetaStart(self, value):
        self.calculationParameters.thetaStart = validatedStringToFloat(value)

    def setThetaStep(self, value):
        self.calculationParameters.thetaStep = validatedStringToFloat(value)

    def setThetaStop(self, value):
        self.calculationParameters.thetaStop = validatedStringToFloat(value)

    def setOutputFilename(self, filename):
        if filename:
            self.calculationParameters.outputFilename = filename


class ProjectionCalculator_thread(QtCore.QThread):
    def __init__(self, parent):
        super(ProjectionCalculator_thread, self).__init__()
        self.parent = parent
    def run(self):
        self.parent.calculateProjections()
if __name__ == "__main__":
    import sys

    # Startup the application
    app = QtWidgets.QApplication(sys.argv)

    # app.setStyle('plastique')
    app.setStyle('mac')

    # Create the GUI
    GENFIRE_ProjectionCalculator = ProjectionCalculator()

    # Render the GUI
    GENFIRE_ProjectionCalculator.show()

    sys.exit(app.exec_())

